---------------------------------------------------------------------------------
--
-- Usage: none!
--
-- This script recreates the tables in Moab's database. It is currently only used 
-- programmatically.
--
--   Copyright (C) 2007-2016 by Adaptive Computing Enterprises, Inc. All Rights Reserved.
--
---------------------------------------------------------------------------------
DROP TABLE IF EXISTS Moab;
DROP TABLE IF EXISTS mcheckpoint;
DROP TABLE IF EXISTS ObjectType;
DROP TABLE IF EXISTS GeneralStats;
DROP TABLE IF EXISTS NodeStats;
DROP TABLE IF EXISTS NodeStatsGenericResources;
DROP TABLE IF EXISTS GenericMetrics;
DROP TABLE IF EXISTS EventType;
DROP TABLE IF EXISTS Events;

CREATE TABLE  `mcheckpoint` (   `object_type` varchar(64) NOT NULL,   `object_name` varchar(64) NOT NULL,   `timestamp` bigint(20) NOT NULL default '0',   `data` mediumtext NOT NULL,   PRIMARY KEY  (`object_type`,`object_name`) );
CREATE TABLE Moab (
        Version FLOAT(6)
);

CREATE TABLE ObjectType (
        ID INTEGER,
        Name VARCHAR(64),
        PRIMARY KEY (ID)
);

CREATE TABLE EventType (
        ID INTEGER,
        Name VARCHAR(64),
        PRIMARY KEY (ID)
);

CREATE TABLE GeneralStats (
        ObjectType INTEGER,
        ObjectName VARCHAR(64),
        Time INTEGER UNSIGNED,
        StatDuration INTEGER,
        IterationCount INTEGER,
        IntervalStatCount INTEGER,
        IntervalStatDuration INTEGER,
        TotalJobCount INTEGER,
        JobsSubmitted INTEGER,
        JobsActive INTEGER,
        JobsEligible INTEGER,
        JobsIdle INTEGER,
        JobsCompletedByQOS INTEGER,
        JobsQueued INTEGER,
        JobsPreempted INTEGER,
        TotalJobAccuracy DOUBLE,
        TotalNodeLoad DOUBLE,
        TotalNodeMemory DOUBLE,
        TotalNodeCount INTEGER,
        TotalAllocatedNodeCount INTEGER,
        TotalActiveNodeCount INTEGER,
        TotalActiveProcCount INTEGER,
        TotalConfiguredProcCount INTEGER,
        TotalAvailableCapacity INTEGER,
        ProcSecondsDedicated DOUBLE,
        ProcSecondsExecuted DOUBLE,
        ProcSecondsRequested DOUBLE,
        ProcSecondsUtilized DOUBLE,
        ProcHoursPreempted DOUBLE,
        ProcHoursSubmitted DOUBLE,
        ProcHoursQueued DOUBLE,
        ProcHoursAvailable DOUBLE,
        ProcHoursDedicated DOUBLE,
        ProcHoursSuccessful DOUBLE,
        ProcHoursUtilized DOUBLE,
        DiskSecondsAvailable DOUBLE,
        DiskSecondsDedicated DOUBLE,
        DiskSecondsUtilized DOUBLE,
        DiskSecondsUtilizedInstant DOUBLE,
        MemSecondsAvailable DOUBLE,
        MemSecondsDedicated DOUBLE,
        MemSecondsUtilized DOUBLE,
        AvgBypass INTEGER,
        MaxBypass INTEGER,
        TotalBypass INTEGER,
        AvgXFactor DOUBLE,
        MaxXFactor DOUBLE,
        AvgQueueTime DOUBLE,
        MaxQueueTime INTEGER,
        TotalQueueTime INTEGER,
        TotalRequestedWalltime DOUBLE,
        TotalExecutionWalltime DOUBLE,
        MinEfficiency DOUBLE,
        MinEfficiencyDuration DOUBLE,
        StartedJobCount INTEGER,
        StartedJobsQueueTime INTEGER,
        StartedJobsProcCount INTEGER,
        StartedJobsXFactor DOUBLE,
        TotalNodeWeightedXFactor DOUBLE,
        TotalNodeWeightedJobAccuracy DOUBLE,
        QOSCredits DOUBLE,
        AppBacklog DOUBLE,
        AppLoad DOUBLE,
        AppResponseTime DOUBLE,
        AppThroughput DOUBLE,
        TotalXFactor DOUBLE,
        Other VARCHAR(256),
        PRIMARY KEY (ObjectType, ObjectName, Time),
        FOREIGN KEY (ObjectType) REFERENCES ObjectType (ID)
);

CREATE TABLE GenericMetrics (
        ObjectType INTEGER,
        ObjectName VARCHAR(64),
        GMetricName VARCHAR(64),
        Time INTEGER UNSIGNED,
        GMetricValue DOUBLE,
        PRIMARY KEY (ObjectType, ObjectName, GMetricName, Time),
        FOREIGN KEY (ObjectType, ObjectName, Time) references GeneralStats(ObjectType, ObjectName, Time)
);

CREATE TABLE NodeStats (
        NodeID VARCHAR(64),
        Time INTEGER UNSIGNED,
        NodeState Integer,
        IterationCount INTEGER,
        IntervalStatCount INTEGER,
        IntervalStatDuration INTEGER,
        InternalNodeCategory VARCHAR(64),
        ExternalNodeCategory VARCHAR(64),
        TotalQueueTime INTEGER UNSIGNED,
        TotalXFactor DOUBLE,
        ProcsAvailable INTEGER,
        ProcsConfigured INTEGER,
        ProcsDedicated INTEGER,
        MaxMemoryUsed INTEGER,
        MinMemoryUsed INTEGER,
        TotalMemoryUsed INTEGER,
        StartJC INTEGER,
        FailureJC INTEGER,
        AvgCPULoad DOUBLE,
        MaxCPULoad DOUBLE,
        MinCPULoad DOUBLE,
        Category INTEGER,
        RCategory INTEGER,
        Other VARCHAR(256),
        CPUClock INTEGER,
        CPUMaxClock INTEGER,
        CPUMinClock INTEGER,
        PRIMARY KEY (NodeID, Time)
);

CREATE TABLE NodeStatsGenericResources (
        NodeID VARCHAR(64),
        GResName VARCHAR(64),
        Time INTEGER UNSIGNED,
        GenericResourcesAvailable INTEGER,
        GenericResourcesConfigured INTEGER,
        GenericResourcesDedicated INTEGER,
        PRIMARY KEY (NodeID, GResName, Time),
        FOREIGN KEY (NodeID, Time) references NodeStats(NodeID, Time)
);

CREATE TABLE Events (
        ID INTEGER,
        ObjectType INTEGER,
        EventType INTEGER,
        EventTime INTEGER UNSIGNED,
        ObjectName VARCHAR(64),
	Name VARCHAR(64),
        Description VARCHAR(1024),
        PRIMARY KEY (ID)
);


  CREATE INDEX EVENTS_NAME on Events (ObjectName);
  CREATE INDEX EVENTS_TIME on Events (EventTime);
  CREATE INDEX EVENTS_ETYPE on Events (EventType);
  CREATE INDEX EVENTS_OTYPE on Events (ObjectType);
  CREATE INDEX EVENTS_ONAME on Events (ObjectName);

 INSERT INTO ObjectType (Name,ID) VALUES ("Acct",1);
 INSERT INTO ObjectType (Name,ID) VALUES ("AM",2);
 INSERT INTO ObjectType (Name,ID) VALUES ("Class",3);
 INSERT INTO ObjectType (Name,ID) VALUES ("Cluster",4);
 INSERT INTO ObjectType (Name,ID) VALUES ("Group",5);
 INSERT INTO ObjectType (Name,ID) VALUES ("Job",6);
 INSERT INTO ObjectType (Name,ID) VALUES ("Node",7);
 INSERT INTO ObjectType (Name,ID) VALUES ("Par",8);
 INSERT INTO ObjectType (Name,ID) VALUES ("QOS",9);
 INSERT INTO ObjectType (Name,ID) VALUES ("Rsv",13);
 INSERT INTO ObjectType (Name,ID) VALUES ("RM",14);
 INSERT INTO ObjectType (Name,ID) VALUES ("Sched",15);
 INSERT INTO ObjectType (Name,ID) VALUES ("SRsv",16);
 INSERT INTO ObjectType (Name,ID) VALUES ("Sys",17);
 INSERT INTO ObjectType (Name,ID) VALUES ("TNode",18);
 INSERT INTO ObjectType (Name,ID) VALUES ("Trig",19);
 INSERT INTO ObjectType (Name,ID) VALUES ("User",20);
 INSERT INTO ObjectType (Name,ID) VALUES ("CJob",23);
 INSERT INTO ObjectType (Name,ID) VALUES ("GRes",30);
 INSERT INTO ObjectType (Name,ID) VALUES ("Gmetric",31);
 INSERT INTO ObjectType (Name,ID) VALUES ("Stats",39);
 INSERT INTO ObjectType (Name,ID) VALUES ("TJob",42);
 INSERT INTO ObjectType (Name,ID) VALUES ("Paction",43);
 INSERT INTO ObjectType (Name,ID) VALUES ("VM",45);
 INSERT INTO ObjectType (Name,ID) VALUES ("VPC",47);
 INSERT INTO ObjectType (Name,ID) VALUES ("JGroup",48);
 INSERT INTO ObjectType (Name,ID) VALUES ("VC",50);

 INSERT INTO EventType (Name,ID) VALUES ("[NONE]",0);
 INSERT INTO EventType (Name,ID) VALUES ("GEVENT",1);
 INSERT INTO EventType (Name,ID) VALUES ("JOBCANCEL",2);
 INSERT INTO EventType (Name,ID) VALUES ("JOBEND",3);
 INSERT INTO EventType (Name,ID) VALUES ("JOBFAILURE",4);
 INSERT INTO EventType (Name,ID) VALUES ("JOBHOLD",5);
 INSERT INTO EventType (Name,ID) VALUES ("JOBMIGRATE",6);
 INSERT INTO EventType (Name,ID) VALUES ("JOBMODIFY",7);
 INSERT INTO EventType (Name,ID) VALUES ("JOBPREEMPT",8);
 INSERT INTO EventType (Name,ID) VALUES ("JOBREJECT",9);
 INSERT INTO EventType (Name,ID) VALUES ("JOBRESUME",10);
 INSERT INTO EventType (Name,ID) VALUES ("JOBSTART",11);
 INSERT INTO EventType (Name,ID) VALUES ("JOBSUBMIT",12);
 INSERT INTO EventType (Name,ID) VALUES ("JOBUPDATE",13);
 INSERT INTO EventType (Name,ID) VALUES ("JOBVARSET",14);
 INSERT INTO EventType (Name,ID) VALUES ("JOBVARUNSET",15);
 INSERT INTO EventType (Name,ID) VALUES ("NODEDOWN",16);
 INSERT INTO EventType (Name,ID) VALUES ("NODEFAILURE",17);
 INSERT INTO EventType (Name,ID) VALUES ("NODEUP",18);
 INSERT INTO EventType (Name,ID) VALUES ("NODEUPDATE",19);
 INSERT INTO EventType (Name,ID) VALUES ("NOTE",20);
 INSERT INTO EventType (Name,ID) VALUES ("QOSVIOLATION",21);
 INSERT INTO EventType (Name,ID) VALUES ("RMDOWN",22);
 INSERT INTO EventType (Name,ID) VALUES ("RMUP",23);
 INSERT INTO EventType (Name,ID) VALUES ("RSVCANCEL",24);
 INSERT INTO EventType (Name,ID) VALUES ("RSVCREATE",25);
 INSERT INTO EventType (Name,ID) VALUES ("RSVEND",26);
 INSERT INTO EventType (Name,ID) VALUES ("RSVMODIFY",27);
 INSERT INTO EventType (Name,ID) VALUES ("RSVSTART",28);
 INSERT INTO EventType (Name,ID) VALUES ("SCHEDCOMMAND",29);
 INSERT INTO EventType (Name,ID) VALUES ("SCHEDFAILURE",30);
 INSERT INTO EventType (Name,ID) VALUES ("SCHEDMODIFY",31);
 INSERT INTO EventType (Name,ID) VALUES ("SCHEDPAUSE",32);
 INSERT INTO EventType (Name,ID) VALUES ("SCHEDRECYCLE",33);
 INSERT INTO EventType (Name,ID) VALUES ("SCHEDRESUME",34);
 INSERT INTO EventType (Name,ID) VALUES ("SCHEDSTART",35);
 INSERT INTO EventType (Name,ID) VALUES ("SCHEDSTOP",36);
 INSERT INTO EventType (Name,ID) VALUES ("TRIGCREATE",37);
 INSERT INTO EventType (Name,ID) VALUES ("TRIGEND",38);
 INSERT INTO EventType (Name,ID) VALUES ("TRIGFAILURE",39);
 INSERT INTO EventType (Name,ID) VALUES ("TRIGSTART",40);
 INSERT INTO EventType (Name,ID) VALUES ("TRIGTHRESHOLD",41);
 INSERT INTO EventType (Name,ID) VALUES ("VMCREATE",42);
 INSERT INTO EventType (Name,ID) VALUES ("VMDESTROY",43);
 INSERT INTO EventType (Name,ID) VALUES ("VMMIGRATE",44);
 INSERT INTO EventType (Name,ID) VALUES ("VMPOWERON",45);
 INSERT INTO EventType (Name,ID) VALUES ("VMPOWEROFF",46);
 INSERT INTO EventType (Name,ID) VALUES ("NODEMODIFY",47);
 INSERT INTO EventType (Name,ID) VALUES ("NODEPOWEROFF",48);
 INSERT INTO EventType (Name,ID) VALUES ("NODEPOWERON",49);
 INSERT INTO EventType (Name,ID) VALUES ("NODEPROVISION",50);
 INSERT INTO EventType (Name,ID) VALUES ("ALLSCHEDCOMMAND",51);
 INSERT INTO EventType (Name,ID) VALUES ("AMDELETE",52);
 INSERT INTO EventType (Name,ID) VALUES ("AMEND",53);
 INSERT INTO EventType (Name,ID) VALUES ("AMQUOTE",54);
 INSERT INTO EventType (Name,ID) VALUES ("AMSTART",55);
 INSERT INTO EventType (Name,ID) VALUES ("RMPOLLEND",56);
 INSERT INTO EventType (Name,ID) VALUES ("RMPOLLSTART",57);
 INSERT INTO EventType (Name,ID) VALUES ("SCHEDCYCLEEND",58);
 INSERT INTO EventType (Name,ID) VALUES ("SCHEDCYCLESTART",59);
 INSERT INTO EventType (Name,ID) VALUES ("JOBCHECKPOINT",60);
 INSERT INTO EventType (Name,ID) VALUES ("AMCREATE",61);
 INSERT INTO EventType (Name,ID) VALUES ("AMUPDATE",62);
 INSERT INTO EventType (Name,ID) VALUES ("AMPAUSE",63);
 INSERT INTO EventType (Name,ID) VALUES ("AMRESUME",64);
 INSERT INTO EventType (Name,ID) VALUES ("AMMODIFY",65);
 INSERT INTO EventType (Name,ID) VALUES ("AMCONTINUE",66);
 INSERT INTO EventType (Name,ID) VALUES ("NODEREMOVE",67);
 INSERT INTO EventType (Name,ID) VALUES ("NODEADD",68);
