#!/usr/bin/perl
###############################################################################
#
#   Copyright (C) 2010-2016 by Adaptive Computing Enterprises, Inc. All Rights Reserved.
#
###############################################################################
use strict;
use warnings;
use File::Copy;
use POSIX;
use IPC::Open3;
use Getopt::Long;
use IO::Select;
use FileHandle;
use FindBin;

use lib "$FindBin::Bin/lib/perl5";
use Moab::Build::Common;
use Moab::Build::Process;
use Moab::Build::DBInstallerInfo;

sub usage{
  println("USAGE: install.sqlite3.pl [--noprompt] [--overwrite-by-default] Moab-Home-Directory Moab_Library-Location");
  exit(-1);
}

my ($noprompt, $overwrite_by_default) = (0,0);

GetOptions(
  "noprompt" => \$noprompt,
  "overwrite-by-default", \$overwrite_by_default,
);

my ($moabHomeDir, $moabLibLocation) = @ARGV;

usage() unless defined($moabHomeDir);

$moabLibLocation ||= "lib";
verifyDirectory($moabHomeDir, "Specified Moab home directory");
verifyDirectory($moabLibLocation, "Specified Moab library directory");


my $lib_path_var;

if (`uname -a` =~ /AIX/) {
  $lib_path_var = 'LIBPATH';
}
else {
  $lib_path_var = 'LD_LIBRARY_PATH';
}
my $ldPath = $ENV{$lib_path_var};

if($ldPath) {
  $ENV{$lib_path_var} = "$moabLibLocation:$ldPath";
}
else {
  $ENV{$lib_path_var} = "$moabLibLocation";
}

my ($driver, $dsn, $dbSetupFunction) = Moab::Build::DBInstallerInfo::getInstallInfo("SQLite3", $moabHomeDir, undef, $moabLibLocation, $noprompt, $overwrite_by_default);

$dbSetupFunction->();
1;
__END__

=head1 NAME

=head1 SYNOPSIS

=head1 DESCRIPTION

=head2 EXPORT

=head2 EXPORT_OK

=head1 SEE ALSO

Additional information can be found at http://www.adaptivecomputing.com/

=head1 AUTHOR

Adaptive Computing, Inc., E<lt>http://www.adaptivecomputing.com/E<gt>

=head1 COPYRIGHT AND LICENSE

Copyright (C) 2010-2016 by Adaptive Computing Enterprises, Inc. All Rights Reserved.

=cut
