package Moab::Build::Common;
###############################################################################
#
#   Copyright (C) 2010-2014 by Adaptive Computing Enterprises, Inc. All Rights Reserved.
#
###############################################################################
use base 'Exporter';
use POSIX;
use strict;
use warnings;

our @EXPORT = qw(
println
error
confirm
confirmOverwrite
verifyDirectory
checked_unlink
getExeFullPath
);

sub println{
  print shift;
  print "\n";
}

sub checked_unlink {
  unlink(@_) or error("failed to delete one of the following:\n\t" . join("\n\t", @_) );
}

sub error {
  println("ERROR:    " . shift);
  exit(-1);
}

sub getExeFullPath {
  my($exe, $dir) = @_;
  my $fullExe = "$dir/$exe";
  chomp($fullExe);
  error("could not find exe $exe in directory $dir") unless -e $fullExe;
  return $fullExe;
}

sub confirm {
  my ($prompt, $default) = @_;
  my $input = "";
  my $promptSuffix = "[Y|N]" . (defined($default) ? (" (Default: " . ($default ? "Y" : "N" ) . ")") : "" );
  do {
    print("$prompt$promptSuffix");
    chomp($input = <STDIN>);
  }
  until ($input =~/^([Y|N])\s*$/i || (defined($default) && $input =~/^\s*$/) );
  return lc (substr ($input, 0, 1) ) eq 'y';
}

sub confirmOverwrite {
  my ($string, $noprompt, $overwrite_by_default) = @_;

  if ($noprompt) {
    return $overwrite_by_default;
  }
  else {
    return confirm($string, $overwrite_by_default);
  }
}

sub verifyDirectory {
  my ($dir, $desc) = @_;
  my $full_desc = "$desc \`$dir'";
  error("$full_desc does not exist ") unless -e $dir;
  error("$full_desc is not a directory ") unless -d $dir;
  error("$full_desc is not readable by this script") unless -r $dir;
  error("$full_desc is not writeable by this script") unless -w $dir;
}


1;
__END__

=head1 NAME

=head1 SYNOPSIS

=head1 DESCRIPTION

=head2 EXPORT

=head2 EXPORT_OK

=head1 SEE ALSO

Additional information can be found at http://www.adaptivecomputing.com/

=head1 AUTHOR

Adaptive Computing, Inc., E<lt>http://www.adaptivecomputing.com/E<gt>

=head1 COPYRIGHT AND LICENSE

Copyright (C) 2010-2014 by Adaptive Computing Enterprises, Inc. All Rights Reserved.

=cut
