package SQLite3;
###############################################################################
#
#   Copyright (C) 2010-2014 by Adaptive Computing Enterprises, Inc. All Rights Reserved.
#
###############################################################################
use Moab::Build::Common;

sub driver {
  my($moabHomeDir, $libInstallDir, $moabLibLocation) = @_;
  my $driverName = driverName();
  my $suffix = '.so';

  my $suffix = '.a' if `uname -a` =~ /AIX/;

  return  <<"  DONE"
[$driverName]
Description		= $driverName ODBC Driver
Driver		= $libInstallDir/libsqlite3odbc$suffix
Setup		= /$libInstallDir/libsqlite3odbc$suffix
  DONE
  ;
}

sub dsn {
  my($moabHomeDir, $libInstallDir, $moabLibLocation) = @_;
  my $sqliteFile = "$moabHomeDir/moab.db";
  return <<"  DONE"
[ODBC] 
Driver=SQLite3
Description=SQLite database test
Database=$sqliteFile
# optional lock timeout in milliseconds
Timeout=2000
  DONE
  ;
}

sub installDB {
  my($moabHomeDir, $libInstallDir, $moabLibLocation, $noprompt, $overwrite_by_default) = @_;
  my $schemaFile =  "buildutils/create.db.all.sql";
  my $sqliteFile = "$moabHomeDir/moab.db";
  if(-e $sqliteFile && !confirmOverwrite("Database $sqliteFile already exists. Do you want to overwrite it? This will destroy any data currently stored in it!", $noprompt, $overwrite_by_default)) {
    println("Not overwriting database file $sqliteFile");
  }
  else {
    my $sqliteExe = getExeFullPath('sqlite3', $moabLibLocation);
    system("$sqliteExe $sqliteFile < $schemaFile") and error("There was an error creating $sqliteFile with sqlite3");
    print("created sqlite3 database file $sqliteFile\n");
  }
}

sub uninstallDB {
  my($moabHomeDir, $libInstallDir, $moabLibLocation, $noprompt, $overwrite_by_default) = @_;
  my $sqliteFile = "$moabHomeDir/moab.db";
  if(-e $sqliteFile) {
    if(confirmOverwrite("Do you wish to remove the sqlite database $sqliteFile?", $noprompt, $overwrite_by_default)) {
      checked_unlink($sqliteFile);
      println("deleted database $sqliteFile");
    }
    else {
      println("not deleting database $sqliteFile");
    }
  }
}

sub driverName {
  return "SQLite3";
}

package Moab::Build::DBInstallerInfo;
use Moab::Build::Common;

sub getInstallInfo {
  my($driverName, $moabHomeDir, $libInstallDir, $moabLibLocation, $noprompt, $overwrite_by_default) = @_;
  my $driver, $dsn, $dbSetupFunction; 


  if($driverName eq "SQLite3") {
    $driver = SQLite3::driver($moabHomeDir, $libInstallDir, $moabLibLocation, $noprompt, $overwrite_by_default);
    $dsn = SQLite3::dsn($moabHomeDir, $libInstallDir, $moabLibLocation, $noprompt, $overwrite_by_default);
    $dbSetupFunction = sub { SQLite3::installDB($moabHomeDir, $libInstallDir, $moabLibLocation, $noprompt, $overwrite_by_default) };
    return ($driver, $dsn, $dbSetupFunction); 
  }
  else{
    error("Could not get installation info for unrecognized or unsupported driver $driverName.");
  }
}

sub getUninstallInfo {
  my($driverName, $moabHomeDir, $libInstallDir, $noprompt, $overwrite_by_default) = @_;
  if($driverName eq "SQLite3") {
    return (SQLite3::driverName, sub { SQLite3::uninstallDB($moabHomeDir, $libInstallDir, $moabLibLocation, $noprompt, $overwrite_by_default) });
  }
  else{
    error("Could not get uninstallation infor for unrecognized or unsupported driver $driverName.");
  }
}

sub getDriverName {
  my($name) = (@_);
  #alias SQLite and SQLite3;
  $name = 'SQLite3' if($name eq 'SQLite');
  return $name;
}

sub driverInstalled {
  my($driver, $exePrefix) = (@_);
  $exePrefix = ($exePrefix) ? "$exePrefix/" : "";
  my $exe = $exePrefix . "odbcinst";
  my $installedDrivers = `$exe -q -d`;

  return $installedDrivers =~ /\[$driver\]/m;
}


1;
__END__

=head1 NAME

=head1 SYNOPSIS

=head1 DESCRIPTION

=head2 EXPORT

=head2 EXPORT_OK

=head1 SEE ALSO

Additional information can be found at http://www.adaptivecomputing.com/

=head1 AUTHOR

Adaptive Computing, Inc., E<lt>http://www.adaptivecomputing.com/E<gt>

=head1 COPYRIGHT AND LICENSE

Copyright (C) 2010-2014 by Adaptive Computing Enterprises, Inc. All Rights Reserved.

=cut
