#!/usr/bin/perl
###############################################################################
#
#   Copyright (C) 2010-2014 by Adaptive Computing Enterprises, Inc. All Rights Reserved.
#
###############################################################################

package Moab::Build::Process;
use strict;
use warnings;
use IO::Select;
use IPC::Open3;

sub new {
  my $class = shift;
  my $self = {};
  bless $self, $class;
  $self->__begin(@_);
  return $self;
}


sub __begin {
  my $self = shift;
  my($cmd, @args) = @_;
  my ($in, $out, $err, $errOut);
  $self->{childpid} = open3($in, $out, $err, $cmd, @args);
  die("failed to start execution $cmd") unless defined($in);

  $self->{in} = $in, $self->{out} = $out, $self->{'err'} = $err;
  $self->{selector} = IO::Select->new($self->{out});
  $self->{selector}->add($self->{'err'}) if defined($err);
}

sub write {
  my ($self, $str) = @_;
  my $in = $self->{in};
  print $in $str;
}

sub __strings {
  my ($outOutput, $errOutput) = @_;
  if(length($outOutput) + length($errOutput) == 0) {
    return undef;
  }
  else {
    return ($outOutput, $errOutput);
  }
}
sub sysread {
  my ($self, $numChars) = @_;
  my ($outOutput, $errOutput) = ("", "");
  $numChars ||= 1000000;
  my @ready = $self->{selector}->can_read;
  foreach my $handle(@ready) {
    my $tmp = "";
    my $charsRead = sysread $handle, $tmp, $numChars;
    if($charsRead == 0) {
      $self->{selector}->remove($handle);
    }
    elsif(defined($self->{'err'}) && $handle == $self->{'err'}) {
      $errOutput .= $tmp;
    }
    else {
      $outOutput .= $tmp;
    }
  }
  return __strings($outOutput, $errOutput);
}

sub end {
  my ($self) = @_;
  close $self->{in};
  my($outOutput, $errOutput) = ("", "");
  while(my ($outChars, $errChars) = $self->sysread)  {
    last unless defined($outChars) || defined($errChars);
    $outOutput .= $outChars, $errOutput .= $errChars;
  }
  close $self->{out};
  close $self->{'err'} if defined($self->{'err'});
  waitpid($self->{childpid}, 0);

  return __strings($outOutput, $errOutput);
}

1;
__END__

=head1 NAME

=head1 SYNOPSIS

=head1 DESCRIPTION

=head2 EXPORT

=head2 EXPORT_OK

=head1 SEE ALSO

Additional information can be found at http://www.adaptivecomputing.com/

=head1 AUTHOR

Adaptive Computing, Inc., E<lt>http://www.adaptivecomputing.com/E<gt>

=head1 COPYRIGHT AND LICENSE

Copyright (C) 2010-2014 by Adaptive Computing Enterprises, Inc. All Rights Reserved.

=cut
