#################################################################################
# Configuration file for moab tools
#
#   Copyright (C) 2006-2016 by Adaptive Computing Enterprises, Inc. All Rights Reserved.
#
################################################################################
use constant DIE  => 0;
use constant WARN => 1;

# Important:  Moab::Tools must be included in the calling script
# before this config file so that homeDir is properly set.
our ($homeDir);

# Set the PATH to include directories for moab client commands -- mjobctl, etc.
$SAVEDPATH = $ENV{PATH};
$ENV{PATH} = "/opt/moab/bin:$ENV{PATH}";

# Specify level of log detail
$logLevel = 3;

# List the resource manager types that will require job script translations
# Valid types include: PBS, LSF, SLURM, SGE, LCRM
@resourceManagers = ();

# Default columns to show for query clients
$jobShow =
  "JobId,UserName,State,Processors,Nodes,WallclockDuration,SubmitTime,StartTime,EndTime";

# How often should the emulators poll clients
$pollInterval = 5;

# Time to wait for ssh command before assuming a hang
$timeout = 5;

# Shell prompt seen on nodes (used in some emulation scripts)
$loginPrompt = "(>|%|#|\\\$) \$";

################################################################################
# unsupportedOption($option, $warnOnly, $verbose)
# Fail with an error for unsupported options
################################################################################
sub unsupportedOption
{
    my ($option, $warnOnly, $verbose) = @_;

    my $message =
      "This option ($option) is not currently supported.\nType '$0 --help' for available options.\nReport problems to <moab-support\@adaptivecomputing.com>.\n";

    if ($warnOnly)
    {
        if ($verbose) { logWarn("WARNING: $message"); }
        else { logPrint("WARNING: $message"); }
    }
    else
    {
        logDie($message);
    }
}

# "1;" Required for file inclusion
1;
__END__

=head1 NAME

=head1 SYNOPSIS

=head1 DESCRIPTION

=head2 EXPORT

=head2 EXPORT_OK

=head1 SEE ALSO

Additional information can be found at http://www.adaptivecomputing.com/

=head1 AUTHOR

Adaptive Computing, Inc., E<lt>http://www.adaptivecomputing.com/E<gt>

=head1 COPYRIGHT AND LICENSE

Copyright (C) 2006-2016 by Adaptive Computing Enterprises, Inc. All Rights Reserved.

=cut
