package Moab::RMConfig::MOAB;
#################################################################################
# Configuration file for moab tools
#
#   Copyright (C) 2006-2014 by Adaptive Computing Enterprises, Inc. All Rights Reserved.
#
################################################################################

#use strict;
#use warnings;
use Moab::Tools;
use constant DIE  => 0;
use constant WARN => 1;
use Exporter;
our @ISA = qw(Exporter);
our @EXPORT_OK = qw(DIE WARN $SAVEDPATH unsupportedOption);

# Default values - Please use update-rmconfig to change these values
#DEFAULT########################################################################
our $PATH = "/opt/moab/bin:$ENV{PATH}";
our $checkjob = "checkjob";
our $checknode = "checknode";
our $jobShow = "JobId,UserName,State,Processors,Nodes,WallDuration,SubmitTime,StartTime,EndTime";
our $logLevel = "3";
our $mdiag = "mdiag";
our $mjobctl = "mjobctl";
our $msub = "msub";
our @resourceManagers = ();
our $showq = "showq";
our $showres = "showres";
push @EXPORT_OK, qw( $PATH $checkjob $checknode $jobShow $logLevel $mdiag $mjobctl $msub @resourceManagers $showq $showres );
#DEFAULT#END####################################################################

$SAVEDPATH = $ENV{PATH};
$ENV{PATH} = $PATH;

# Set paths as necessary -- these can be short names if PATH is included above
#$mjobctl   = "mjobctl";
#$mdiag     = "mdiag";
#$msub      = "msub";
#$showq     = "showq";
#$checkjob  = "checkjob";
#$checknode = "checknode";
#$showres   = "showres";
#
# List the resource manager types that will require job script translations
# Valid types include: PBS, LSF, SLURM, SGE, LCRM
#@resourceManagers = ();
#
# Default columns to show for query clients
#$jobShow = "JobId,UserName,State,Processors,Nodes,WallDuration,SubmitTime,StartTime,EndTime";


################################################################################
# unsupportedOption($option, $warnOnly, $verbose)
# Fail with an error for unsupported options
################################################################################
sub unsupportedOption ($$$)
{
    my ($option, $warnOnly, $verbose) = @_;

    my $message = "This option ($option) is not currently supported.\n" .
                  "Type '$0 --help' for available options.\n" .
                  "Report problems to <moab-support\@adaptivecomputing.com>.\n";

    if ($warnOnly)
    {
        if ($verbose) { logWarn("WARNING: $message"); }
        else          { logPrint("WARNING: $message"); }
    }
    else
    {
        logDie($message);
    }
}

1; # Required -- Do not remove
__DATA__
$PATH
$logLevel
$mjobctl
$mdiag
$msub
$showq
$checkjob
$checknode
$showres
@resourceManagers
$jobShow

__END__

=head1 NAME

=head1 SYNOPSIS

=head1 DESCRIPTION

=head2 EXPORT

=head2 EXPORT_OK

=head1 SEE ALSO

Additional information can be found at http://www.adaptivecomputing.com/

=head1 AUTHOR

Adaptive Computing, Inc., E<lt>http://www.adaptivecomputing.com/E<gt>

=head1 COPYRIGHT AND LICENSE

Copyright (C) 2006-2014 by Adaptive Computing Enterprises, Inc. All Rights Reserved.

=cut
