#!/usr/bin/python

#THIS SOFTWARE SCRIPT IS PROVIDED BY ADAPTIVE COMPUTING ENTERPRISES, INC. "AS IS" 
#AND IS FOR REFERENCE USE ONLY.  ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT 
#NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR 
#PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL ADAPTIVE COMPUTING ENTERPRISES, INC. 
#BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
#DAMAGES ARISING FROM THE USE THEREOF.

import sys, getopt, string, subprocess, os
import shutil

class Config:
  def __init__(self):
    self.start = False
    self.restart = False
    self.entries = {}
    self.homeDir = os.environ.get('MOABHOMEDIR','/opt/moab/')
    if self.homeDir.endswith('/') == False:
      self.homeDir += str("/")


# ATTENTION:  The ipmiuser and ipmipass must be setup correctly
#     1) ipmiuser is the username necessary for ipmitool to authenticate to nodes.
#     2) ipmipass is a full path to a password file that contains the password for ipmitool to authenticate.
#
#     To verify, try:
#
#       ipmitool -I lan -H <ipmi host> -U <ipmiuser> -f <ipmi password file> chassis power status
#
#     Desired Results:  "Chassis Power is on"
#     BAD Results:       "Unable to get Chassis Power Status"   Indicates failure communicating.


    self.ipmiuser = "ADMIN"
    self.ipmipass = self.homeDir + "passfile.txt"

# CUSTOMER ATTENTION:  The <MoabHomeDir>/node-bmc.txt file must contain a list of nodes.
#   Example of node-bmc.txt file as follows:
#
#   node1 node1_ipmi 
#   node2 10.1.5.15       # NOTE:  IP Address may be used.

    self.bmcaddrmap = "node-bmc.txt"    #Name of IPMI nodes file.  May be changed.

    self.ipmifile = "ipmicache.gm"      #Temp file that stores the cluster query information
    self.pollInterval = 120
    self.daemon = self.homeDir + "tools/ipmi/__mon.ipmi.py"
    self.pidFile = self.homeDir + "spool/ipmimon.pid"

    # Verify OpenIPMI has been installed.  Prerequisites will exit upon failure.
    self.VerifyPrerequisites()

    try:
      opts, args = getopt.getopt(sys.argv[1:],"skrh",["start","stop","restart","help"])
    except getopt.GetoptError:
      print 'ipmi.mon.py -s|start -stop|k -restart|r -help|?'
      sys.exit(2)

    for opt, arg in opts:
      if opt in ("-h", "?", "--help"):
        print 'ipmi.mon.py -s|start -stop|k -restart|r -help|?'
        sys.exit(0)
      elif opt in ("-s", "--start"):
        self.start = True
      elif opt in ("-k", "--stop"):
        self.kill_pid()
        sys.exit(0)

      elif opt in ("-r", "--restart"):
        self.restart = True


# Scan through each node and issue ipmi commands 
    try:
      f = open(self.homeDir + self.bmcaddrmap, 'r') 
    except IOError:
      sys.stderr.write('ERROR: Unable to read bmcaddrmap file : ' + self.homeDir + self.bmcaddrmap + '\n')
      sys.exit(-1)

    for line in f.readlines():
      line.strip()
      if line[0] == '#':
        continue

      try:
        left, right = string.split(line)
      except ValueError:   #ValueError can be caused by a blank line, so just continue to next line.
        continue

      try:
        self.entries[left].append(right)
      except KeyError:
        self.entries[left] = right

  def kill_pid(self):

    try:
      f = open(self.pidFile)
      lines = f.readlines()
      f.close()
    except IOError,AttributeError:
      return 

#  remove the pid file
    if (len(lines) > 0):
      os.remove(self.pidFile)

      cmd = "kill -15 " + lines[0]
      subprocess.Popen(cmd, shell=True, stdout=subprocess.PIPE, stderr=subprocess.STDOUT)

  def VerifyPrerequisites (self):

    # Verify ipmitool has been installed by obtaining the ipmitool version string 
    p = subprocess.Popen("ipmitool -V", shell=True, stdout=subprocess.PIPE, stderr=subprocess.STDOUT)
    results = p.stdout.readlines()

    # Verify the version is found.
    if results[0].find("version") < 0:
      print "ERROR 'Unable to execute ipmitool'"
      exit(-1)

