#!/usr/bin/python

#THIS SOFTWARE SCRIPT IS PROVIDED BY ADAPTIVE COMPUTING ENTERPRISES, INC. "AS IS" 
#AND IS FOR REFERENCE USE ONLY.  ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT 
#NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR 
#PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL ADAPTIVE COMPUTING ENTERPRISES, INC. 
#BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
#DAMAGES ARISING FROM THE USE THEREOF.

# Modification for Cray XC systems
# =============================================================================
# THIS SCRIPT HAS BEEN MODIFIED TO SERVE AS A "REFERENCE" SCRIPT FOR USE WITH
# CRAY XC SYSTEMS.  IT WILL REQUIRE MODIFICATION TO WORK WITH A SPECIFIC CRAY.
# =============================================================================

import sys, string, subprocess, os

class Config:
  def __init__(self):
    self.start = False
    self.restart = False
    self.entries = {}
    self.homeDir = "/opt/moab/"         # Moab home directory; change if necessary.

# ATTENTION:  The ipmiuser and ipmipass must be set up correctly.
#     1) ipmiuser is the user name necessary for ipmitool to authenticate to nodes.
#     2) ipmipass is a full path to a password file that contains the password for ipmitool to authenticate.
#
#     To verify, try:
#
#       ipmitool -I lan -H <ipmi host> -U <ipmiuser> -f <ipmi password file> chassis power status
#
#     Desired Results:  "Chassis Power is on"
#     BAD Results:      "Unable to get Chassis Power Status" indicates failure communicating.

    self.ipmiuser = "ADMIN"
    self.ipmipass = self.homeDir + "passfile.txt"

# Modification for Cray XC systems
# =============================================================================
# For Cray XC capmc commands, these variables (or other authentication) must be
# in place.  Note you MUST replace the "https://snake-smw.us.cray.com:8443" URL
# with the URL appropriate to your Cray system.
# =============================================================================
    os.environ["OS_CACERT"] = self.homeDir + "ssl/certificate_authority.crt"
    os.environ["OS_KEY"] = self.homeDir + "ssl/client.key"
    os.environ["OS_CERT"]= self.homeDir + "ssl/client.crt"
    os.environ["OS_SERVICE_URL"] = "https://snake-smw.us.cray.com:8443"
# =============================================================================

# CUSTOMER ATTENTION:  The <MoabHomeDir>/node-bmc.txt file must contain a list of nodes.
#   Example of node-bmc.txt file as follows:
#
#   node1 node1_ipmi 
#   node2 10.1.5.15       # NOTE:  IP Address may be used.

    self.bmcaddrmap = "node-bmc.txt"    #Name of IPMI nodes file.  May be changed.

    self.ipmifile = "ipmicache.gm"      #Temp file that stores the cluster query information
    self.pollInterval = 120
    self.daemon = self.homeDir + "tools/ipmi/cray/__mon.ipmi.py"
    self.pidFile = self.homeDir + "spool/ipmimon.pid"

    # Modification for Cray XC systems
    # =============================================================================
    # The IPMI "node host name/BMC IP address mapping" text file is not needed by
    # the capmc command; hence, the file is not used and therefore the following
    # code reading the file is commented out.
    # =============================================================================
    # Load the IPMI nodes file that maps node names to IPMI IP addresses
    #    for line in open(self.homeDir + self.bmcaddrmap, 'r').readlines():
    #      line.strip()
    #      if line[0] == '#':
    #        continue
    #
    #      left, right = string.split(line)
    #      try:
    #        self.entries[left].append(right)
    #      except KeyError:
    #        self.entries[left] = right
    # =============================================================================

  def kill_pid(self):

    try:
      f = open(self.pidFile)
      lines = f.readlines()
      f.close()
    except IOError,AttributeError:
      print "error reading " + self.pidFile
      return 

#  remove the pid file
    if (len(lines) > 0):
      os.remove(self.pidFile)

      cmd = "kill -15 " + lines[0]
      subprocess.Popen(cmd, shell=False, stdout=subprocess.PIPE, stderr=subprocess.STDOUT)

