#!/usr/bin/python
  
import sys, termios, atexit
from select import select
from time import sleep
import zmq
import os

#-----------------------------------------------
# Keyboard functions
#-----------------------------------------------

# save the terminal settings
fd = sys.stdin.fileno()
new_term = termios.tcgetattr(fd)
old_term = termios.tcgetattr(fd)

# new terminal setting unbuffered
new_term[3] = (new_term[3] & ~termios.ICANON & ~termios.ECHO)

# switch to normal terminal
def set_normal_term():
    termios.tcsetattr(fd, termios.TCSAFLUSH, old_term)

# switch to unbuffered terminal
def set_curses_term():
    termios.tcsetattr(fd, termios.TCSAFLUSH, new_term)

def getch():
    return sys.stdin.read(1)

def kbhit():
    dr,dw,de = select([sys.stdin], [], [], 0)
    return dr <> []

#-----------------------------------------------
# GetFunctionName
#-----------------------------------------------

def GetFunctionName(line):
    sections = line.split(",")
    for section in sections:
        if "Function" in section:
            return section.split(":")[1]

#-----------------------------------------------
# GetDuration
#-----------------------------------------------

def GetDurationText(line):
    sections = line.split(",")
    for section in sections:
        if "Duration" in section:
            tuples = section.split(":")
            return tuples[1].split("\"")[0]

#-----------------------------------------------
# PrintFunctions
#-----------------------------------------------

def PrintFunctions(functionNames, levels):
    spaces = ""
    for n in range(0, levels):
        sys.stdout.write(spaces + functionNames[n] + "        \n")
        spaces = spaces + "  "
        
    for n in range(0, levels):
      sys.stdout.write("\x1b[A") # Move the cursor up one

#-----------------------------------------------
# Main
#-----------------------------------------------

tabLevel = 0
functionNames = []

atexit.register(set_normal_term)
set_curses_term()

os.system('clear');
context = zmq.Context()
sock = context.socket(zmq.SUB)
sock.setsockopt(zmq.SUBSCRIBE, 'telemetry.timing')
sock.connect('tcp://localhost:5566')
while True:
    try:
        if kbhit():
            ch = str(getch())
            if (ch == "q" or ch == "Q"):
                os.system('clear');
                break

        message = sock.recv(zmq.NOBLOCK)
        if "MTimer.c" in message:
            lines = message.splitlines()
            for line in lines:
                if "TimeIn" in line:
                    # increment the tab level, and set the subcalls for this level to false
                    functionNames.append(GetFunctionName(line))
                    spaces = ""
                    for n in range(0, tabLevel):
                        spaces = spaces + " "
                    sys.stdout.write(spaces + functionNames[tabLevel] + "\n")

                    tabLevel = tabLevel + 1

                elif "Duration" in line:
                    if tabLevel > 0:
                        tabLevel = tabLevel - 1

                    # clear the line
                    spaces = ""
                    for n in range(0, tabLevel):
                        spaces = spaces + " "
                    functionName = ""
                    if len(functionNames) > 0:
                        functionName = functionNames.pop()
                    for n in range(0, len(functionName)):
                        spaces = spaces + " "
                    sys.stdout.write("\x1b[A")
                    sys.stdout.write("\r" + spaces + "\r")

    except (zmq.ZMQError):
        # wait for 50ms before trying again
        sleep(0.05)
        pass
