#!/usr/bin/python
# ====================================================================
# Each class should be written with the follwing in mind
#  init() - Initial call to setup the object
#  run()  - Ran only if init passes
#  offline_mode() - Gather logs and files.
#  diag() - Diagnostics
# ====================================================================
# ====================================================================
# Import from libraries and validate Python version
# ====================================================================
import os
import shutil
import threading
import ACgeneral as ACgeneral


class ACcray:
    """Common base class for all Cray diag pieces"""
    def __init__(self):
        self.is_running = False
        self.pid = 0
        self.manually_detected = False
        self.log_dir = ""
        self.commands_detected = False
        self.commands_active = False
        self.output_dir = "/tmp/"
        self.commands_to_run = {"apstat -n -vvvv 2>&1": "apstat_-n_-vvvv.txt", "apstat -r 2>&1": "apstat_-r.txt",
                                "apstat -c 2>&1": "apstat_-c.txt",
                                "echo \"<?xml version='1.0'?><BasilRequest protocol='1.3' method='QUERY' "
                                "type='INVENTORY'></BasilRequest>\" | /usr/bin/apbasil": "inventory.txt"
                                }

    # ====================================================================
    #  Initialize - Initial call to setup the object
    # ====================================================================
    def init(self, output_dir):
        self.output_dir = output_dir

    # ====================================================================
    # Run - Ran only if init passes
    # ====================================================================
    def run(self):
        if not os.path.exists(self.output_dir+"cray"):
            os.makedirs(self.output_dir+"cray")
        # Look for commands
        self.commands_detected = ACgeneral.cmd_exists("apstat")
        if not self.commands_detected:
            return 1
        else:
            for cmd, out_file in self.commands_to_run.iteritems():
                c_thread = threading.Thread(target=ACgeneral.run_and_save,
                                            args=(cmd, out_file, self.output_dir+"cray/"))
                c_thread.name = "cray"
                c_thread.daemon = True
                c_thread.start()
        return 0

    # ====================================================================
    # Diag - Diagnostics
    # ====================================================================
    def diag(self):
        diag_print = "\n================= CRAY DIAGS ================="
        diag_print += "\nCray Running: " + str(self.is_running)
        diag_print += "\nCray PID: " + str(self.pid)
        diag_print += "\nCray manually detected: " + str(self.manually_detected)
        diag_print += "\nCray log dir: " + self.log_dir
        diag_print += "\nCray commands detected: " + str(self.commands_detected)
        diag_print += "\nCray: commands active: " + str(self.commands_active)
        diag_print += "\nCray: output dir: " + self.output_dir
        return diag_print
