#!/usr/bin/python
# ====================================================================
# Each class should be written with the follwing in mind
#  init() - Initial call to setup the object
#  run()  - Ran only if init passes
#  offline_mode() - Gather logs and files. 
#  diag() - Diagnostics
# ====================================================================

# ====================================================================
# Import from libraries and validate Python version
# ====================================================================
import sys
import os
import re
import subprocess
import shutil
import threading
import ACgeneral as ACgeneral


class ACmam:
    """Common base class for all MAM diag pieces"""
    def __init__(self):
        self.is_running = False
        self.pid = 0
        self.home_dir = "/opt/mam/"
        self.manually_detected = False
        self.log_dir = "/opt/mam/log/"
        self.commands_detected = False
        self.commands_active = False
        self.commands_to_run = {"gbalance": "gbalance.txt",
                                "glsaccount": "glsaccount.txt",
                                "glsalloc": "glsalloc.txt",
                                "glsfund": "glsfund.txt",
                                "glslien": "glslien.txt",
                                "glsrate": "glsrate.txt",
                                "glsuser": "glsuser.txt",
                                "glstrans -s `date -d \"1 day ago\" '+%Y-%m-%d'`": "glstrans_-s.txt"
                                }

    # ====================================================================
    #  Initialize - Initial call to setup the object
    # ====================================================================
    def init(self, output_directory):
        self.output_dir = output_directory

    # ====================================================================
    # Run - Ran only if init passes
    # ====================================================================
    def run(self):
        if os.path.exists(self.home_dir):
            shutil.copytree(self.home_dir, self.output_dir+"mam", ignore=shutil.ignore_patterns('*core*'))
            # Gather ls -hR for directory listing and size
            ACgeneral.run_and_save("ls -lhR " + self.home_dir, "mam_ls_-lhR.txt", self.output_dir)

        self.gather_mam_diags()

        return 0

    # ====================================================================
    # Offline mode
    # ====================================================================
    def offline_mode(self):
        if os.path.exists(self.home_dir):
            shutil.copytree(self.home_dir, self.output_dir+"mam", ignore=shutil.ignore_patterns('*core*'))

    # ====================================================================
    # Diag - Diagnostics
    # ====================================================================
    def diag(self):
        diag_print = "\n================= MAM DIAGS ================="
        diag_print += "\nMAM running:" + str(self.is_running)
        diag_print += "\nMAM pid:" + str(self.pid)
        diag_print += "\nMAM home:" + self.home_dir
        diag_print += "\nMAM Manually detected:" + str(self.manually_detected)
        diag_print += "\nMAM logdir:" + self.log_dir
        diag_print += "\nMAM commands detected:" + str(self.commands_detected)
        diag_print += "\nMAM commands active:" + str(self.commands_active)

        return diag_print

    # ====================================================================
    # Gather mam - Diagnostics
    # ====================================================================
    def gather_mam_diags(self):
        for cmd, out_file in self.commands_to_run.iteritems():
            match = re.search(r'glstrans', cmd,)
            if match:
                if ACgeneral.cmd_exists("glstrans"):
                    mam_thread = threading.Thread(target=ACgeneral.run_and_save,
                                                  args=(cmd, out_file, self.output_dir+"mam/"))

                    mam_thread.name = "mam"
                    mam_thread.daemon = True
                    mam_thread.start()
            else:
                if ACgeneral.cmd_exists(cmd):
                    mam_thread = threading.Thread(target=ACgeneral.run_and_save,
                                                  args=(cmd, out_file, self.output_dir+"mam/"))
                    mam_thread.name = "mam"
                    mam_thread.daemon = True
                    mam_thread.start()


        return 0


