#!/usr/bin/python
# ====================================================================
# Each class should be written with the follwing in mind
#  init() - Initial call to setup the object
#  run()  - Ran only if init passes
#  diag() - Diagnostics
# ====================================================================

# ====================================================================
# Import from libraries and validate Python version
# ====================================================================
import re
import subprocess
import threading
import ACgeneral as ACgeneral
import sys
import os
import shutil
import datetime


class ACmoab:
    """Common base class for all Moab diag pieces"""
    def __init__(self, today_date_time, full_logging, core_dumps):
        self.cwd = ""
        self.output_dir = "/tmp/"
        self.command_time_out = " --timeout=300"
        self.core_dumps = core_dumps
        self.home_dir = "/opt/moab/"
        self.pid = 0
        self.bin_path = "/opt/moab/bin/"
        self.sbin_path = "/opt/moab/sbin/"
        self.manually_detected = False
        self.log_dir = ""
        self.log_file = ""
        self.full_log_mode = full_logging
        self.etc_path = ""
        self.cmds_timeout = 300
        self.port = "42559"
        self.test_cmd = "showq"
        self.commands_detected = False
        self.commands_active = False
        self.today_date_time = today_date_time
        self.submit_host = False
        moab_cmd_parms = " --timeout="+ str(self.cmds_timeout) + " --port=" +self.port
        self.sbin_commands_to_run = {"moab --about 2>&1": "moab_--about.txt", "msub --about 2>&1": "msub_--about.txt",}
        self.bin_commands_to_run = {"showq -v" + moab_cmd_parms: "showq_-v.txt",
                       "mdiag -a -v" + moab_cmd_parms: "mdiag_-a_-v.txt",
                       "mdiag -R -v" + moab_cmd_parms: "mdiag_-R_-v.txt",
                       "showstats -v" + moab_cmd_parms: "showstats_-v.txt",
                       "mdiag -R -v -v" + moab_cmd_parms: "mdiag_-R_-v_-v.txt",
                       "showq -v --blocking" + moab_cmd_parms: "showq_-v_--blocking.txt",
                       "mdiag -p -v" + moab_cmd_parms: "mdiag_-p_-v.txt",
                       "checkjob -v -v -v ALL" + moab_cmd_parms: "checkjob_-v_-v_-v_ALL.txt",
                       "mdiag -n -v" + moab_cmd_parms: "mdiag_-n_-v.txt",
                       "mdiag -c -v" + moab_cmd_parms: "mdiag_-c_-v.txt",
                       "mdiag -C" + moab_cmd_parms: "mdiag_-C.txt",
                       "mdiag -j -v" + moab_cmd_parms: "mdiag_-j_-v.txt",
                       "mdiag -g -v" + moab_cmd_parms: "mdiag_-g_-v.txt",
                       "mdiag -G -v" + moab_cmd_parms: "mdiag_-G_-v.txt",
                       "mdiag -q -v" + moab_cmd_parms: "mdiag_-q_-v.txt",
                       "mdiag -u -v" + moab_cmd_parms: "mdiag_-u_-v.txt",
                       "mdiag -t -v" + moab_cmd_parms: "mdiag_-t_-v.txt",
                       "mdiag -T -v" + moab_cmd_parms: "mdiag_-T_-v.txt",
                       "mdiag -r -v" + moab_cmd_parms: "mdiag_-r_-v.txt",
                       "mdiag -s -v" + moab_cmd_parms: "mdiag_-s_-v.txt",
                       "mdiag -S -v" + moab_cmd_parms: "mdiag_-S_-v.txt",
                       "mdiag -l" + moab_cmd_parms: "mdiag_-l.txt",
                       "mshow" + moab_cmd_parms: "mshow.txt",
                       "mcredctl -q policies user:ALL" + moab_cmd_parms: "mcredctl_-q_policies_user:ALL.txt",
                       "showres -v" + moab_cmd_parms: "showres_-v.txt",
                       "showres -n -v" + moab_cmd_parms: "showres_-n_-v.txt",
                       "showq --loglevel=9 2>&1" + moab_cmd_parms: "showq_--loglevel=9.txt",
                       "mdiag -f -v" + moab_cmd_parms: "mdiag_-f_-v.txt",
                       "mschedctl -l vpc" + moab_cmd_parms: "mschedctl_-l_vpc.txt",
                       "mvmctl -q ALL" + moab_cmd_parms: "mvmctl_-q_ALL.txt",
                       "checknode -v -v ALL" + moab_cmd_parms: "checknode_-v_-v_ALL.txt",
                       "showq -c -v" + moab_cmd_parms: "showq_-c_-v.txt",
                       "mnodectl -q wiki ALL" + moab_cmd_parms: "mnodectl_-q_wiki_ALL.txt",
                       "mdiag -p -t" + moab_cmd_parms: "mdiag_-p_-t.txt", "showq --about 2>&1": "showq_--about.txt",
                       "showconfig -v" + moab_cmd_parms: "showconfig_v.txt",
                       "checkjob --about 2>&1": "checkjob_--about.txt",
                       "changeparam --about 2>&1": "changeparam_--about.txt",
                       "mjobctl --about 2>&1": "mjobclt_--about.txt", 
                       "mdiag --about 2>&1": "mdiag_--about.txt",
                       "mrsvctl --about 2>&1": "mrsvctl_--about.txt", 
                       "mshow --about 2>&1": "mshow_--about.txt",
                       "mvmctl --about 2>&1": "mvmctl_--about.txt", 
                       "runjob --about 2>&1": "runjob_--about.txt",
                       "showbf --about 2>&1": "showbf_--about.txt", 
                       "showstart --about 2>&1": "showstart_--about.txt",
                       "msub --about 2>&1": "msub_--about.txt"}

        self.sbin_commands_to_run = {"moab --about 2>&1": "moab_--about.txt", 
                                     "moabd --about 2>&1": "moabd_--about.txt"}
       
    # ====================================================================
    #  Initialize - Initial call to setup the object
    # ====================================================================
    """ 
      Takes:
            func_c as cmd_exists in ACgeneral
    """ 
    def init(self, output_dir, moab_timeout):
        self.output_dir = output_dir
        self.cmds_timeout = int(moab_timeout)
        # Try to make the moab tmp outpur dir
        try:
            os.makedirs(self.output_dir+"moab_logs")
        except:
            print("I was unable to make the output direcotry: " + self.output_dir + "moab_logs")
            sys.exit(1)
        # Capture cwd to save.
        self.cwd = os.getcwd()
        self.check_path_for_moab_cmds(ACgeneral.cmd_exists)
        if self.commands_detected:
            self.validate_moabcmds_are_active(self.test_cmd)
        self.check_for_moab_pid()
        self.determin_cmd_path()
        self.determin_moab_home()
        self.determin_moab_service_port()

    # ====================================================================
    # Run - Runs only if init passes
    # ====================================================================
    def run(self):
        self.gather_moab_diagnostics()
        self.gather_moab_files()
        # Dynamic log dir checking from moab.cfg
        self.get_moab_cfg_log_dir()
        self.gather_moab_logs()
        self.check_for_backtraces()
        if self.core_dumps is not None:
            self.gather_moab_cores()

    # ====================================================================
    # Gather offline files
    # ====================================================================
    def offline_mode(self):
        self.gather_moab_files()
        self.get_moab_cfg_log_dir()
        self.gather_moab_logs()
        self.check_for_backtraces()
        if self.core_dumps is not None:
            self.gather_moab_cores()

    # ====================================================================
    # Diag - Diagnostics
    # ====================================================================
    def diag(self):
        diag_print = "\n================= MOAB DIAGS ================="
        diag_print += "\nMoab home:" + self.home_dir 
        diag_print += "\nMoab PID:" + str(self.pid)
        diag_print += "\nMoab bin:" + self.bin_path
        diag_print += "\nMoab sbin:" + self.sbin_path
        diag_print += "\nMoab manually detected:" + str(self.manually_detected)
        diag_print += "\nMoab logdir:" + self.log_dir
        diag_print += "\nMoab etc dir" + self.etc_path
        diag_print += "\nMoab timeout:" + str(self.cmds_timeout)
        diag_print += "\nMoab port:" + str(self.port)
        diag_print += "\nMoab detected:" + str(self.commands_detected)
        diag_print += "\nMoab active:" + str(self.commands_active)
        diag_print += "\nMoab cwd:" + self.cwd

        return diag_print

    # ====================================================================
    # Called from init() - Checks if commands are in the path
    # ====================================================================
    def check_path_for_moab_cmds(self, func_c):
        # This checks sbin commands
        if func_c("moab"):
            # This checks for bin commands
            self.commands_detected = True
        if func_c("showq"):
            self.commands_detected = True
        else:
            self.commands_detected = False
    
    # ====================================================================
    # Called from init() - Check if moab commands work
    # ====================================================================  
    def validate_moabcmds_are_active(self, cmd_to_run):
        p = subprocess.Popen(cmd_to_run, stdout=subprocess.PIPE, stderr=subprocess.PIPE, shell=True)
        output, error = p.communicate()

        if output:
            match = re.search(r'active', output)
            if match:
                self.commands_detected = True
                self.commands_active = True
        elif error:
            match = re.search(r'command not found', error)
            if match:
                print "The Moab commands are not in the path!"
                self.commands_detected = False
                self.commands_active = False
            match = re.search(r'ERROR', error)
            if match:
                print "Moab appears to be shutdown or unresponsive. "
                print "I will attempt to gather offline diagnostics..."
                self.commands_detected = True
                self.commands_active = False

    # ====================================================================
    # Called from init() - Check for a moab pid 
    # ==================================================================== 
    def check_for_moab_pid(self):
        return_code, cmd_output = ACgeneral.run_and_return_output("ps hf -opid -C moab | awk '{ print $1; exit }'")
        if return_code == 0 and cmd_output == '':
            print ("Moab process not found.")
        elif return_code == 0:
            self.pid = cmd_output.rstrip('\n')

    # ====================================================================
    # Called from init() - Check for a moab command path 
    # ====================================================================
    """
     This function needs to handle the following situations.
     1) If we have already detected Moab commands then simply parse path
     2) If the commands are not in the path but we see a Moab PID walk /proc/<PID>. 
     3) If this is a submithost
    """ 
    def determin_cmd_path(self):
        if self.pid > 0 and self.commands_detected and self.commands_active:
            # Try parsing sbin dir out from which.
            return_code, cmd_output = ACgeneral.run_and_return_output("/bin/bash/ -c 'which moab'")
            if return_code == 0 and cmd_output == '':
                print ("I was not able to find sbin dir for moab.")
            elif return_code == 0: 
                # self.sbin_path = re.sub('moab', "", cmd_output )
                self.sbin_path = cmd_output[:-5]
                self.sbin_path = self.sbin_path.rstrip()
            # Try parsing bin dir out from which.
            return_code, cmd_output = ACgeneral.run_and_return_output("/bin/bash/ -c 'which showq'")
            if return_code == 0 and cmd_output == '':
                print ("I was not able to find bin dir for Moab client commands.")
            elif return_code == 0:
                self.bin_path = re.sub('showq', "", cmd_output)
                self.bin_path = self.bin_path.rstrip()

        if self.pid > 0 and not self.commands_detected:
            print ("Looking for Moab client command path.")
            print ("Export your Moab binarie and client command path to skip auto detection.")
            proc_path = "/proc/" + str(self.pid)
            proc_path = proc_path.rstrip()
            proc_path += str("/")
            proc_path_to_test = "file -0 " + proc_path + "exe | awk '{print $5}'"
            return_code, bin_dir_tmp = ACgeneral.run_and_return_output(proc_path_to_test)
            if return_code == 0:
                sbin_dir = bin_dir_tmp
                sbin_dir = re.sub('`', "", sbin_dir)
                sbin_dir = re.sub("'", "", sbin_dir)
                sbin_dir = sbin_dir.strip()
                sbin_dir = sbin_dir[:-4]
                self.sbin_path = sbin_dir
                self.bin_path = re.sub("sbin", "bin", sbin_dir)
                self.bin_path = self.bin_path.strip()
                # Test for bindir commands
                self.validate_moabcmds_are_active(self.bin_path+"showq")

        if self.pid == 0 and self.commands_detected and self.commands_active:
            print ("If this is a submit host please re-run on your HPC server.")
            print ("I will go ahead and gather client command output.")
            self.submit_host = True

    # ====================================================================
    # Called from init() - Try to find and or set MOABHOMEDIR 
    # ====================================================================
    """
     Cases to check for
     1) Check if MOABHOMEDIR is in the path
     2) Parse /proc/<PID> if there is not env for MOABHOMEDIR.
    """
    def determin_moab_home(self):
        if 'MOABHOMEDIR' in os.environ.keys():
            self.home_dir = os.environ['MOABHOMEDIR']
            if self.home_dir[-1:] == "/":
                self.etc_path = self.home_dir + "etc/"
            else:
                self.home_dir += "/"
                self.etc_path = self.home_dir + "etc/"
        else:
            if self.pid == 0:
                # Dynamically try to find the homedir.
                print "I do not see Moab running so I will try to determin the path manually."
                m_path_list = ["/opt/moab/", "/var/spool/moab/", "/opt/MOAB/", "/opt/Moab/", "/var/spool/Moab/",
                               "/var/spool/MOAB/", "/var/spool/moab/", "/var/spool/opt/moab/", "/opt/moab/default/"]
                for id_name in m_path_list:
                    if os.path.isdir(id_name):
                        self.home_dir = id_name
                        # Set Moab home so we can run client commands.
                        os.environ['MOABHOMEDIR'] = self.home_dir
                        self.home_dir += "etc/"
                        return 0
            else:
                # Walk proc for Moab home.
                home_path = "/proc/" + str(self.pid)
                home_path = home_path.rstrip()
                home_path += str("/")
                home_path += "cwd/"
                if os.path.isdir(home_path):
                    # Try to pick up Moab home from proc.
                    os.chdir(home_path)
                    return_code, home_dir = ACgeneral.run_and_return_output("pwd -P")
                    if return_code == 0:
                        self.home_dir = str(home_dir)
                        self.home_dir = self.home_dir.rstrip('\n')
                        self.home_dir += "/"
                        self.etc_path = self.home_dir
                        self.home_dir = self.home_dir.replace('etc/', '')
                        os.environ['MOABHOMEDIR'] = self.home_dir
                    os.chdir(self.cwd)
                    return 0
                else:
                    print ("The proc path for Moab disappeared!")
                    sys.exit(1)

    # ====================================================================
    # Called from init() - Try to find Moab service port.
    # ====================================================================
    """
     Cases to check for port number in moab.cfg. 
     Honor $MOABHOMEDIR/etc/moab.cfg over $MOABHOMEDIR/moab.cfg
    """
    def determin_moab_service_port(self):
        if os.path.exists(self.home_dir+"etc/moab.cfg"):
            # Read moab.cfg and grab SCHEDCFG lines without # in it.
            schedcfg_return = ACgeneral.run_and_return_output("grep SCHEDCFG " + self.home_dir +
                                                              "etc/moab.cfg | grep SERVER | grep -v '#'")
            tmp_str = schedcfg_return[1]
            # split it and set port
            self.port = ACgeneral.find_between(tmp_str, ":", " ")
            if self.port == "":
                self.port = ACgeneral.find_between(tmp_str, ":", "")
            if self.port == "":
                self.port = ACgeneral.find_between(tmp_str, ":", "\n")
        elif os.path.exists(self.home_dir+"moab.cfg"):
            # Read moab.cfg and grab SCHEDCFG lines without # in it.
            schedcfg_return = ACgeneral.run_and_return_output("grep SCHEDCFG " + self.home_dir +
                                                              "moab.cfg | grep SERVER | grep -v '#'")
            tmp_str = schedcfg_return[1]
            # split it and set port
            self.port = ACgeneral.find_between(tmp_str, ":", " ")
            if self.port == "":
                self.port = ACgeneral.find_between(tmp_str, ":", "")
            if self.port == "":
                self.port = ACgeneral.find_between(tmp_str, ":", "\n")

        if self.port == "":
            print "Moab Port was not set. Using default 42559"
            self.port = "42559"

        return 0

    # ====================================================================
    # Called from init() -> run() - Gather Moab diagnostics.
    # ====================================================================
    """
     Gather client command output for Moab. 
    """
    def gather_moab_diagnostics(self):
        os.chdir(self.output_dir)
        moab_cmd_parms = self.command_time_out
        moab_cmd_parms += " --port=" + self.port
        if self.manually_detected:
            for cmd, out_file in self.bin_commands_to_run.iteritems():
                m_thread = threading.Thread(target=ACgeneral.run_and_save,
                                            args=(self.sbin_path+cmd, out_file, self.output_dir), name="moab")
                m_thread.name = "moab"
                m_thread.daemon = True
                m_thread.start()
            for cmd, out_file in self.sbin_commands_to_run.iteritems():
                m_thread = threading.Thread(target=ACgeneral.run_and_save,
                                            args=(self.sbin_path+cmd, out_file, self.output_dir), name="moab")
                m_thread.name = "moab"
                m_thread.daemon = True
                m_thread.start()

        else:
            for cmd, out_file in self.bin_commands_to_run.iteritems():
                m_thread = threading.Thread(target=ACgeneral.run_and_save,
                                            args=(cmd, out_file, self.output_dir), name="moab")
                m_thread.name = "moab"
                m_thread.daemon = True
                m_thread.start()
            for cmd, out_file in self.sbin_commands_to_run.iteritems():
                m_thread = threading.Thread(target=ACgeneral.run_and_save,
                                            args=(cmd, out_file, self.output_dir), name="moab")
                m_thread.name = "moab"
                m_thread.daemon = True
                m_thread.start()

        # Run ldd on Moab
        ldd_cmd = "none"
        if ACgeneral.cmd_exists("moab"):
            ldd_cmd = "ldd `which moab`"
        elif os.path.isfile(self.sbin_path + "moab"):
            ldd_cmd = "ldd " + self.sbin_path + "moab"
        if ldd_cmd != "none":
            ACgeneral.run_and_save(ldd_cmd, "ldd_moab.txt", self.output_dir)
        if os.path.exists(self.home_dir):
            # Gather ls -hR for directory listing and size
            ACgeneral.run_and_save("ls -lhR " + self.home_dir, "moab_ls_-lhR.txt", self.output_dir)

        proc_path = "/proc/" + str(self.pid)
        proc_path = proc_path.rstrip()
        proc_path += str("/")
        if os.path.exists(proc_path):
            # Gather ls -hR for directory listing and size
            ACgeneral.run_and_save("ls -lhR " + proc_path, "moab_proc_ls_-lhR.txt", self.output_dir)

    # ====================================================================
    # Called from gather_moab_files() - Gather # include files.
    # ====================================================================
    """
     Gather #INCLUDE files. 
    """
    def check_for_pound_include(self):
        if os.path.exists(self.home_dir+"etc/moab.cfg"):
            # Read moab.cfg and grab #INCLUDE
            include_return = ACgeneral.run_and_return_output("grep INCLUDE " +
                                                             self.home_dir + "etc/moab.cfg | grep -v \"##\"")
            tmp_str = include_return[1]
            tmp_str = re.sub(r"#INCLUDE\s+", "", tmp_str)
            tmp_str_tbl = tmp_str.split()
            for words in tmp_str_tbl:
                os.chdir(self.home_dir+"etc/")
                if os.path.isfile(words):
                    shutil.copy(words, self.output_dir)
                    if os.path.isfile(self.home_dir+"etc/"+words):
                        shutil.copy(self.home_dir+"etc/"+words, self.output_dir + "etc/")

        if os.path.exists(self.home_dir+"moab.cfg"):
            # Read moab.cfg and grab #INCLUDE
            include_return = ACgeneral.run_and_return_output("grep INCLUDE " +
                                                             self.home_dir + "moab.cfg | grep -v \"##\"")
            tmp_str = include_return[1]
            tmp_str = re.sub(r"#INCLUDE\s+", "", tmp_str)
            tmp_str_tbl = tmp_str.split()
            for words in tmp_str_tbl:
                os.chdir(self.home_dir)
                if os.path.isfile(words):
                    shutil.copy(words, self.output_dir)
                if os.path.isfile(self.home_dir+words):
                    shutil.copy(self.home_dir+words, self.output_dir)
        return 0

    # ====================================================================
    # Called from init() -> run() - Gather Moab files.
    # ====================================================================
    """
     Gather $MOABHOMEDIR files. 
    """
    def gather_moab_files(self):
        if os.path.exists(self.home_dir):
            os.chdir(self.home_dir)
            if os.path.exists(self.etc_path):
                try:
                    shutil.copytree(self.etc_path, self.output_dir + "etc/", ignore=shutil.ignore_patterns('*core*'))
                except:
                    print ("I was unable to copy " + self.etc_path)
        # Look for moab.cfg
        moab_cfg = self.home_dir + "moab.cfg"
        if os.path.isfile(moab_cfg):
            try:
                shutil.copy(moab_cfg, self.output_dir)
            except:
                print ("I was unable to copy " + moab_cfg)
            # Pound include only.
            self.check_for_pound_include()

        moab_cfg_etc = self.home_dir + "etc/moab.cfg"
        if os.path.isfile(moab_cfg_etc):
            shutil.copy(moab_cfg_etc, self.output_dir)
            # Pound include only.
            self.check_for_pound_include()

        # Check for additional files
        moab_private = self.home_dir + "moab-private.cfg"
        moab_private_etc = self.home_dir + "etc/moab-private.cfg"
        moab_db = self.home_dir + "moab.db"
        moab_db_etc = self.home_dir + "etc/moab.db"
        moab_chkpt = self.home_dir + ".moab.ck"
        moab_chkpt_1 = self.home_dir + ".moab.ck.1"
        moab_chkpt_etc = self.home_dir + "etc/.moab.ck"
        moab_chkpt_etc_1 = self.home_dir + "etc/.moab.ck.1"

        moab_tool_dir = self.home_dir + "tools/"
        moab_stat_dir = self.home_dir + "stats/"
        moab_sbin_dir = self.home_dir + "sbin/"

        # PID map for Moab
        moab_pid_map = "/proc/" + str(self.pid) + "/maps"
        moab_pid_limits = "/proc/" + str(self.pid) + "/limits"        

        # Need to add ACgeneral call here so we don't reuse code
        if os.path.isfile(moab_pid_map):
            try:
                shutil.copy(moab_pid_map, self.output_dir+"/moab_pid_maps")
            except IOError, e:
                print "Unable to copy file. %s" % e
        if os.path.isfile(moab_pid_limits):
            try:
                shutil.copy(moab_pid_limits, self.output_dir+"/moab_pid_limits")
            except IOError, e:
                print "Unable to copy file. %s" % e
        if os.path.isfile(moab_private):
            try:
                shutil.copy(moab_private, self.output_dir)
            except IOError, e:
                print "Unable to copy file. %s" % e
        if os.path.isfile(moab_private_etc):
            try:
                shutil.copy(moab_private_etc, self.output_dir)
            except IOError, e:
                print "Unable to copy file. %s" % e
        if os.path.isfile(moab_db):
            try:
                shutil.copy(moab_db, self.output_dir)
            except IOError, e:
                print "Unable to copy file. %s" % e
        if os.path.isfile(moab_db_etc):
            try:
                shutil.copy(moab_db_et, self.output_dir)
            except IOError, e:
                print "Unable to copy file. %s" % e
        if os.path.isfile(moab_chkpt):
            try:
                shutil.copy(moab_chkpt, self.output_dir)
            except IOError, e:
                print "Unable to copy file. %s" % e
        if os.path.isfile(moab_chkpt_1):
            try:
                shutil.copy(moab_chkpt_1, self.output_dir)
            except IOError, e:
                print "Unable to copy file. %s" % e
        if os.path.isfile(moab_chkpt_etc):
            try:
                shutil.copy(moab_chkpt_etc, self.output_dir)
            except IOError, e:
                print "Unable to copy file. %s" % e
        if os.path.isfile(moab_chkpt_etc_1):
            try:
                shutil.copy(moab_chkpt_etc_1, self.output_dir)
            except IOError, e:
                print "Unable to copy file. %s" % e
        if os.path.exists(moab_tool_dir):
            try:
                shutil.copytree(moab_tool_dir, self.output_dir + "/tools/", ignore=shutil.ignore_patterns('*core*'))
            except IOError, e:
                print "Unable to copy file. %s" % e
        if os.path.exists(moab_sbin_dir):
            try:
                shutil.copy(moab_sbin_dir + "moab", self.output_dir)
            except IOError, e:
                print "Unable to copy file. %s" % e

    # ====================================================================
    # Gather #Logs for non standard moab log path
    # ====================================================================
    def get_moab_cfg_log_dir(self):
        """
        :return:
        """
        comment = re.compile('^#')
        os.chdir(self.home_dir)
        # Looks for a custom logdir
        log_dir = None
        if os.path.isfile(self.home_dir+"etc/moab.cfg"):
            results = ACgeneral.run_and_return_output("grep LOGDIR " + self.home_dir + "etc/moab.cfg")
            results = results[1]
            results = results.rstrip()
            results = results.split('\n')
            if len(results[0]) != 0:
                for i in results:
                    invalid = comment.match(i)
                    if invalid is None:
                        log_dir = i.split()
                        log_dir = log_dir[1] + "/"
        elif os.path.isfile(self.home_dir+"moab.cfg"):
            results = ACgeneral.run_and_return_output("grep LOGDIR moab.cfg")
            results = results[1]
            results = results.rstrip()
            results = results.split('\n')
            if len(results[0]) != 0:
                for i in results:
                    invalid = comment.match(i)
                    if invalid is None:
                        log_dir = i.split()
                        log_dir = log_dir[1] + "/"
        if log_dir is None:
            log_dir = self.home_dir + "log/"
        self.log_dir = log_dir

        # Looks for the custom LOGFILE
        log_file = None
        if os.path.isfile(self.home_dir+"etc/moab.cfg"):
            results = ACgeneral.run_and_return_output("grep LOGFILE " + self.home_dir + "etc/moab.cfg")
            results = results[1]
            results = results.rstrip()
            results = results.split('\n')
            if len(results[0]) != 0:
                for i in results:
                    invalid = comment.match(i)
                    if invalid is None:
                        log_file = i.split()
                        log_file = log_file[1] 
        elif os.path.isfile(self.home_dir+"moab.cfg"):
            results = ACgeneral.run_and_return_output("grep LOGFILE moab.cfg")
            results = results[1]
            results = results.rstrip()
            results = results.split('\n')
            if len(results[0]) != 0:
                for i in results:
                    invalid = comment.match(i)
                    if invalid is None:
                        log_file = i.split()
                        log_file = log_file[1] 
        self.log_file = log_file

    # ====================================================================
    # Gather Moab Logs
    # ====================================================================
    def gather_moab_logs(self):
        if self.log_file != None:
            if os.path.isfile(self.log_file):
                try:
                    shutil.copy(self.log_file, self.output_dir)
                except IOError, e:
                    print "Unable to gather " + sefl.log_file, e

        if os.path.exists(self.log_dir):
            if self.full_log_mode:
                ACgeneral.cp_file_pattern(self.log_dir+"*", self.output_dir + "moab_logs/")
                return 0
            # Walk the os directory and find file dates and time.
            files_and_dates = ACgeneral.return_file_and_mtime(self.log_dir, "None")
            # Todays
            moab_log_date_format_today = self.today_date_time
            # Yesterdays
            moab_log_date_format_yesterday = str(int(moab_log_date_format_today) - 1)
            # Day before Yesterday
            moab_log_date_format_day_before_yesterday = str(int(moab_log_date_format_today) - 2)

            for file_name, file_mdate in files_and_dates.iteritems():
                if file_mdate >= moab_log_date_format_today or file_mdate >= moab_log_date_format_yesterday \
                        or file_mdate >= moab_log_date_format_day_before_yesterday:
                    # Copy logs that we detected
                    try:
                        shutil.copy(self.log_dir+file_name, self.output_dir+"moab_logs/")
                    except(OSError, IOError) as e:
                        print "Warning: unable to copy log file ", e

        else:
            # Check the moab.cfg for log dir to copy logs
            print "No moab log directory found. Continuing.."
            return 1 
        return 0

    # ====================================================================
    # Check for backtraces in the moab.log file
    # ====================================================================
    def check_for_backtraces(self):
        """
        :param self.output_dir:
        :param self.log_dir:
        :return: Give 1 if nothing happens
        """
        prog_found = ACgeneral.cmd_exists("addr2line")
        if not prog_found:
            return 1
        # grep moab.log for ----Begin Moab Backtrace--- and grab to
        # the end ---End Moab Backtrace--
        # output into a backtrace file under output_dir

        first = "----Begin Moab Backtrace---"
        last = "---End Moab Backtrace--"
        cmd = "grep -h  -A 100 -e '----Begin Moab Backtrace---' "+ self.log_dir + "moab.log"
        cmd2 = "grep  -A 4 \"Begin Moab Backtrace\" /opt/moab/log/moab.log"
        # split on newline
        back_trace_output = ACgeneral.run_and_return_output(cmd)
        back_trace_output = back_trace_output[1]
        back_trace_output = back_trace_output.rstrip()
        back_trace_output = back_trace_output.split('\n')

        backtrace_information = ACgeneral.run_and_return_output(cmd2)
        backtrace_information = backtrace_information[1]
        backtrace_information = backtrace_information.rstrip()
        backtrace_information = backtrace_information.split('\n')

        backtrace_output = ""
        final_backtrace = "==================Begin Backtrace==================\n"
        backtrace_info_text = ""
        if len(back_trace_output[0]) != 0:
            for item in backtrace_information:
                backtrace_info_text += "\n" + item
            final_backtrace += backtrace_info_text
            for item in back_trace_output:
                backtrace_hexpattern = "0x[0-9a-fA-F]+"
                hex_out = re.search(backtrace_hexpattern, item)
                if hex_out:
                    moab_backtrace_hexpattern = "moab"
                    hex_out_moab = re.search(moab_backtrace_hexpattern, item)
                    if hex_out_moab:
                        item_out = ACgeneral.find_between(item, "[", "]")
                        # Make sure we have valid data to run against addr2line
                        if item_out:
                            # addr2line -f --demangle -e $1 $3
                            cmd = "addr2line -f --demangle -e " + self.sbin_path + "moab" + " " + item_out
                            backtrace_output = ACgeneral.run_and_return_output(cmd)
                            final_backtrace = final_backtrace + backtrace_output[1]
            final_backtrace += "\n==================End Backtrace=================="
            ACgeneral.save_output(final_backtrace, "Backtrace.txt", self.output_dir)
        return 0

    # ====================================================================
    # Check for core files.
    # ====================================================================
    def gather_moab_cores(self):
        list_cores = ACgeneral.find_file_from_pattern("*core*", self.home_dir)
        # Try to make the moab_core folder in outpur dir
        try:
            os.makedirs(self.output_dir+"moab_core")
        except:
            print("I was unable to make the output direcotry: " + self.output_dir + "moab_core")
            return 1

        if list_cores and self.core_dumps == "A":
            for cor in list_cores:
                if os.path.isfile(cor):
                    try:
                        shutil.copy(cor, self.output_dir+"moab_core/")
                    except:
                        e = sys.exc_info()[0]
                        print("There was a failure with trying to copy: %s" % e)
        elif list_cores and ACgeneral.str_is_int(self.core_dumps):
            if int(self.core_dumps) == 0:
                return 0
            # Need support for etc/ as well and loop 2x
            files_and_dates_base = ACgeneral.return_file_and_mtime(self.home_dir, "*core*")
            self.copy_moab_cores(files_and_dates_base)
            
    # ====================================================================
    # Copy core files. Called from gather_moab_cores
    # ====================================================================
    def copy_moab_cores(self, files_and_dates):
        tmp_core_dump_count = int(self.core_dumps)
        core_count = 1
        for file_name, file_mdate in files_and_dates.iteritems():
            # Date format
            moab_core_date_format = str(int(self.today_date_time) - tmp_core_dump_count)
            if file_mdate >= moab_core_date_format:
                print "Copying core file: " + file_name
                if os.path.isfile(file_name):
                    # Copy core
                    try:
                        shutil.copy(file_name, self.output_dir+"moab_core/")
                    except:
                        e = sys.exc_info()[0]
                        print ("I was unable to copy " + file_name + " %s" % e)
                    # Grab the backtrace
                    try:
                        ACgeneral.run_and_save("gdb -q -n -ex bt -batch "+self.sbin_path+"moab " +
                                               file_name + " 2>&1 | grep -v 'No stack.'\$",
                                               str(core_count) + "_bt.txt", self.output_dir + "moab_core/")
                        ACgeneral.run_and_save("gdb -q -n -ex 'bt full' -batch " +
                                               self.sbin_path+"moab " + file_name + " 2>&1 | grep -v 'No stack.'\$",
                                               str(core_count) + "_bt_full.txt", self.output_dir + "moab_core/")
                    except:
                        e = sys.exc_info()[0]
                        print("I was unable to gather a backtrace from: " + file_name + " %s" % e)
