#!/usr/bin/python
# ====================================================================
# Each class should be written with the follwing in mind
#  init() - Initial call to setup the object
#  run()  - Ran only if init passes
#  diag() - Diagnostics
# ====================================================================

# ====================================================================
# Import from libraries and validate Python version
# ====================================================================
import os
import shutil


class ACmws:
    """Common base class for all MWS diag pieces"""
    def __init__(self):
        self.is_running = False
        self.pid = 0
        self.home_dir = "/opt/mws/"
        self.log_dir = "/opt/mws/log/"
        self.output_dir = "/tmp/"

    # ====================================================================
    #  Initialize - Initial call to setup the object
    # ====================================================================
    def init(self, output_dir):
        self.output_dir = output_dir

    # ====================================================================
    # Run - Ran only if init passes
    # ====================================================================
    def run(self):
        self.gather_mws_diag()

    # ====================================================================
    # Diag - Diagnostics
    # ====================================================================
    def diag(self):
        diag_print = "\nMWS running: " + str(self.is_running)
        diag_print += "\nMWS PID: " + str(self.pid)
        diag_print += "\nMWS home dir:" + self.home_dir
        diag_print += "\nMWS log dir: " + self.log_dir
        diag_print += "\nMWS output dir: " + self.output_dir
        return diag_print

    # ====================================================================
    # Gather entire viewpoint directory
    # ====================================================================
    def gather_mws_diag(self):
        if os.path.exists(self.home_dir):
            shutil.copytree(self.home_dir, self.output_dir+"mws", ignore=shutil.ignore_patterns('*core*'))
        return 0
