#!/usr/bin/python
# ====================================================================
# Each class should be written with the follwing in mind
#  init() - Initial call to setup the object
#  run()  - Ran only if init passes
#  offline_mode() - Gather logs and files.
#  diag() - Diagnostics
# ====================================================================

# ====================================================================
# Import from libraries and validate Python version
# ====================================================================
import ACgeneral as ACgeneral
import threading
import os
import shutil


class ACos: 
    """Common base class for all OS diag pieces"""
    def __init__(self): 
        self.os_version = ""
        self.output_dir = "/tmp/"
        # OS commands to run
        self.commands_to_run = {"uname -a": {"uname_-a.txt": "uname"},
                                "uname -n": {"uname_-n.txt": "uname"},
                                "hostnamectl": {"hostnamectl.txt": "hostnamectl"},
                                "df": {"df.txt": "df"},
                                "ps aux": {"ps_aux.txt": "ps"},
                                "vmstat 1 4": {"vmstat_1_4.txt": "vmstat"},
                                "date": {"date.txt": "date"},
                                "free -k": {"free_-k.txt": "free"},
                                "mount": {"mount.txt": "mount"},
                                "chkconfig --list 2>&1": {"chkconfig_--list.txt": "chkconfig"},
                                "uptime": {"uptime.txt": "uptime"},
                                "route": {"route.txt": "route"},
                                "iptables-save": {"iptables-save.txt": "iptables"},
                                "ip addr show": {"ip_addr_show.txt": "ip"},
                                "ifconfig": {"ifconfig.txt": "ifconfig"},
                                "netstat": {"netstat.txt": "netstat"},
                                "netstat -nap": {"netstat_-nap.txt": "netstat"},
                                "ss": {"ss.txt": "ss"},
                                "ss -l": {"ss_-l.txt": "ss"},
                                "ss -pl": {"ss_-pl.txt": "ss"},
                                "service --status-all": {"service_--status-all.txt": "service"},
                                "systemctl": {"systemctl_.txt": "systemctl"},
                                "systemctl -l": {"systemctl_-l.txt": "systemctl"},
                                "systemctl -a": {"systemctl_-a.txt": "systemctl"},
                                "systemctl -r": {"systemctl_-r.txt": "systemctl"},
                                "rpm -qa": {"rpm_-qa.txt": "rpm"},
                                "getenforce": {"getenforce.txt": "getenforce"},
                                "sestatus": {"sestatus.txt": "sestatus"},
                                "pstree -p": {"pstree.txt": "pstree"},
                                "env": {"env.txt": "env"},
                                "dmidecode": {"dmidecode.txt": "dmidecode"}
                                }
        # OS files to copy
        self.files_to_save = {"/etc/mtab": "mtab.txt",
                              "/etc/group": "group.txt",
                              "/etc/hosts": "host.txt",
                              "/etc/sysconfig/network": "network.txt",
                              "/etc/hosts.equiv": "hosts.equiv.txt",
                              "/etc/odbc.ini": "odbc.ini.txt",
                              "/etc/odbcinst.ini": "odbcinst.ini.txt",
                              "/etc/sysconfig/selinux": "selinux.txt",
                              "/etc/network/interfaces": "interfaces.txt",
                              "/var/log/tomcat/catalina.out": "catalina.out",
                              "/var/log/tomcat6/catalina.out": "catalina6.out"
                              }
        # Directory to copy
        self.etc_sysconfig_network_scripts = "/etc/sysconfig/network-scripts/"
       
    # ====================================================================
    #  Initialize - Initial call to setup the object
    # ====================================================================
    def init(self, output_directory): 
        self.output_dir = output_directory
        # Validate paths and commands
        # Determin os release
        self.run()

    # ====================================================================
    # Run - Ran only if init passes
    # ====================================================================
    def run(self): 
        # Run valid commands and copy valid files
        self.gather_os_diag()

    # ====================================================================
    # Diag - Diagnostics
    # ====================================================================
    def diag(self): 
        diag_print = "\n================= OS DIAGS ================="
        diag_print += "\nVersion: " + self.os_version
        diag_print += "\nOutput directory:  " + self.output_dir
        return diag_print

    # ====================================================================
    # OS - Diagnostics commands for OS
    # ====================================================================
    def gather_os_diag(self): 
        if not os.path.exists(self.output_dir+"os/"): 
            os.makedirs(self.output_dir+"os")
        # Run os commands
        for cmd, cmd_key in self.commands_to_run.iteritems(): 
            for out_file, base_cmd in cmd_key.iteritems(): 
                if ACgeneral.cmd_exists(base_cmd): 
                    os_thread = threading.Thread(target=ACgeneral.run_and_save,
                                                 args=(cmd, out_file, self.output_dir+"os/"))
                    os_thread.name = "os"
                    os_thread.daemon = True
                    os_thread.start()
        # Copy os files
        for c_file, out_file in self.files_to_save.iteritems(): 
            if os.path.isfile(c_file): 
                try: 
                    shutil.copy(c_file, self.output_dir+"os/")
                except IOError, e: 
                    print "Unable to copy " + c_file, e
