#! /usr/bin/perl -w
#################################################################################
# Usage: support.diag.pl [--logdir=<DIR>] [--include-log-lines=<NUM>] [--diag-torque] [--diag-perl] [--tmpdir=<DIR>] [--noupload] [--noticket] [--ticket-id=<SUPPORT_TICKET_ID>] 
#
# NOTE: This file has been deprecated and has been replaced by the newer file:  support-diag.py
#
#
# This script will run multiple diagnostics, gather information about your Moab configuration,
# and may also package Moab configuration and log files in a convienent tarball that can
# then be sent to Adaptive Computing. This file is intended to make diagnosis of issues or possible bugs
# as simple and convient for customers as possible.
# The new features are:
#  automatically scp tarball to Adaptive Computing;
#  prompt user for trouble ticket number;
#  timouts on all moab client commands;
#  checks for TORQUE and runs the torque diag if found.
#
# To run non-interactive set --noupload and --noticket flags.
#
#   Copyright (C) 2005-2016 by Adaptive Computing Enterprises, Inc. All Rights Reserved.
#
################################################################################
use strict;
use warnings;
use FindBin;
use lib "/opt/moab/lib/perl5",      "/opt/moab/etc";        # Installed
use lib "${FindBin::Bin}/../lib/perl5", "${FindBin::Bin}/../etc";   # Configured
use Getopt::Long;
use Scalar::Util qw(looks_like_number);

my $no_expect = 0;
eval { require "Expect.pm" }
  or $no_expect = 1;
# used to debug expect
#$Expect::Exp_Internal = 1;

use Moab::Tools;    # Required before including config to set $homeDir
BEGIN { require "config.moab.pl"; }

use constant AC_SUPPORT => 'guest\@ssh.adaptivecomputing.com:';
use constant MOAB_CMDS_TIMEOUT => 120;

our ($homeDir);
my ($logLevel, $useXML, $cmdCount, @commandList) = (1, 0, -1, ());
my (
    $help,       $logDir,        $includeLogLines, $result,
    $outDir,     $diagTorque,    $diagPerl,        $format,
    $dryRun,     $commandsToRun, $support_tick_id, $diagSLURM,
    $moab_pound, $moabIsRunning, $moabPID,
);
$moab_pound = '0';
$moabPID = `pgrep moab`;

if( looks_like_number($moabPID) eq 1 )
{
    $moabIsRunning = 0;
}
else
{
    $moabIsRunning  = 1;
}
my ($currentDir, $numErrors, $tarBall, $tmpDir) = ('', 0, '', '/tmp');
my $ticket_number = '';
my $trans_meth = 'no';
my $exp;
# default is to ask for upload and ticket
my $upload = 1;
my $ticket = 1;
# Location of the moab binary
chomp( my $moabBinary = `which moab`);

{                   # Parse Command Line Arguments
    my $usage =
      "Usage: $0 [--include-log-lines=<NUM>] [--diag-torque] [--diag-slurm] [--diag-perl] [--format=xml] [--dry] [--tmpdir=<DIR>] [--noupload] [--noticket] [--ticket-id=<SUPPORT_TICKET_ID>]\n\n";
    GetOptions(
        'c=s'                        => \$commandsToRun,
        'diag-perl'                  => \$diagPerl,
        'diag-torque|with-torque'    => \$diagTorque,
        'diag-slurm|with-slurm'      => \$diagSLURM,
        'dry'                        => \$dryRun,
        'format=s'                   => \$format,
        'help|?'                     => \$help,
        'include-log-lines=i'        => \$includeLogLines,
        'logdir=s'                   => \$logDir,
        'tmpdir=s'                   => \$tmpDir,
        't|ticket-id:s'              => \$support_tick_id,
        'upload!'                    => \$upload,
        'ticket!'                    => \$ticket,
    ) or print $usage and exit 1;
    die $usage if $help;
}

  # no spaces allowed in file names
  $ticket_number = $support_tick_id if defined $support_tick_id;
  $ticket_number =~ s/\s//g;

  _sanity_chk();


{    # Prepare
    # some enhancements
    unless ($useXML)
      {
      # ticket number defaults to ''
      $ticket_number = inp_ticket_num () unless $ticket_number or not $ticket;
      $trans_meth    = inp_file_trans () if $upload;
      }
    $diagTorque = 1 if defined $diagTorque or chk_for_torque ();
    $diagSLURM = 1 if defined $diagSLURM or chk_for_slurm ();
    
    _init ();
    debugPrint(
        "Gathering diagnostic information from Moab @ '$homeDir' - please wait...\n"
    );
    print "<results>" if $useXML && ! $dryRun;
}
{    # Gather Diagnostics

    #Do this before the logs get polluted/rotated by other commands.
    checkForBacktraces();
    gatherSystemDiag();
    gatherMoabcfgAndLogs();
    gatherViewpointDiag();
    checkForPoundInclude();
    gatherMamDiag();
    gatherMwsDiag();
    gatherTorqueDiag() if defined $diagTorque;
    gatherTorqueLogs() if defined $diagTorque;
    gatherSlurmDiag()  if defined $diagSLURM;
    gatherPerlDiag()   if defined $diagPerl;
    gatherMoabDiag(); 
}
{    # Complete Process
    bundlePackage();
    if ($numErrors)
    {
        debugPrint(
            "\nInfo: Completed diagnostics, but with $numErrors error(s)\n"
        );
    }
    else
    {
        debugPrint("\nCompleted diagnostics successfully with no errors.\n");
    }

    debugPrint("\nDiagnostics were gathered and placed in file \"$tarBall\"\n\n");

  unless ($dryRun)
    {
    # transfer the file
    if ($trans_meth eq 'scp')
      {
      upload_file ();
      }
    elsif ($trans_meth eq 'no')
      {
      print
        "Completed creating the diagnostics.\n\n"
        unless $useXML or not $upload;
      } # END elsif ($trans_meth eq 'no'...)
    else
      {
      my $inter_trans_err = interact_upload_file ();

      if ($inter_trans_err)
        {
        print
          "Please send this file \"$tarBall\" to your Adaptive Computing support contact.\n\n";
        }

      } # END else [ if ($trans_meth eq 'scp'...)]

    } # END unless ($dryRun)

    print "</commands>\n"                               if $useXML && $dryRun;
    print "<file Name=\"$tarBall\"></file></results>\n" if $useXML && ! $dryRun;

}
exit 0;

###############################################################################
#
sub _sanity_chk
  {
    # Verify that the Moab commands are in the PATH
    `which showq`;
    die
      "WARNING: cannot find the Moab commands (showq, mjobctl, mdiag, etc.) in the current PATH!\n"
      if (! $useXML && $?);

    # Change to Moab's home directory for remainder of script
    $currentDir = `pwd`;
    chomp $currentDir;
    chdir($homeDir) or die "Cannot change to Moab Home Directory: $homeDir";

    return;
  }
#
###############################################################################
#
sub _init
{
    # Extract all numerical commands in list
    if ($commandsToRun)
    {
        @commandList = grep {/\d+/} split /,/, $commandsToRun;
    }

    # Check for special formatting options
    if ($format && ($format =~ /^xml$/i))
    {
        $useXML   = 1;
        $logLevel = 0;
    }

    # Change to Moab's home directory for remainder of script
    $currentDir = `pwd`;
    chomp $currentDir;
    chdir($homeDir) or die "Cannot change to Moab Home Directory: $homeDir";

    # Determine hostname
    my $hostname = `hostname`;
    chomp $hostname;
    $hostname = 'moab-diag' if $?;

    # Determine timestamp
    my $timestamp = `date +\"\%Y-\%m-\%d_\%H-\%M-\%S\"`;
    chomp $timestamp;
    $timestamp = 'now' if $?;

    # Make sure logdir is set
    $logDir = "log" unless $logDir;

    # Create special directory and file to store diagnostics
    my $packageName = "$hostname.$timestamp";
    $packageName = "$packageName.$ticket_number" if
      $ticket_number;

    $outDir = "$tmpDir/Moab/support.diag.pl/$packageName";
    if    ($tmpDir) { $tarBall = "$tmpDir/$packageName.tar.gz" }
    elsif ($ENV{'PWD'}) { $tarBall = "$ENV{'PWD'}/$packageName.tar.gz" }
    else                { $tarBall = "$currentDir/$packageName.tar.gz" }
    execCmd("mkdir -p $outDir");
}
#
#------------------------------------------------------------------------------
#
sub chk_for_torque
  {
  my $found_torque = 0;

  my $torquePID = `pgrep pbs_server`;
  if ( looks_like_number($torquePID) eq 1)
  {
    $found_torque = 1;
    return $found_torque;
  }
  else
  {
    open(MDIAG, "mdiag -R |") or return 0;
    while (<MDIAG>)
      {
      if (/Type:\s+(PBS|TORQUE)/)
        {
        $found_torque = 1;
        last;
        }
      } # END while (<MDIAG>)
  
    close MDIAG;

    return $found_torque;
  }
  } # END sub chk_for_torque
#
#------------------------------------------------------------------------------
#
sub chk_for_slurm
  {
  my $found_slurm = 0;

  my $slurmPID = `pgrep slurmd`;
  if ( looks_like_number($slurmPID) eq 1)
  {
    $found_slurm = 1;
    return $found_slurm;
  }
  else
  {
    open(MDIAG, "mdiag -R |") or return 0;
    while (<MDIAG>)
      {
      if (/Type:\s+(WIKI:SLURM|WIKI|SLURM)/)
        {
        $found_slurm = 1;
        last;
        }
      } # END while (<MDIAG>)

    close MDIAG;

    return $found_slurm;
  }
  } # END sub chk_for_slurm
#
#------------------------------------------------------------------------------
#
sub interact_upload_file
  {
  my $rc;
  my $trans_err_msg =
    "\n\nSorry, could not transfer the file \"$tarBall\" to Adaptive Computing support.\n\n";

  # check for file trans exec
  $rc = system('which scp >/dev/null');
  $rc = $rc >> 8;

  if ($rc)
    {
    print $trans_err_msg;
    return $rc;
    }

  unless ($dryRun)
    {
    print "Starting the scp file transfer process.\n\n";
    print "You need to type in the password 'hello' (no quotes)\n";

    $rc = system("scp $tarBall " . AC_SUPPORT);
    $rc = $rc >> 8;

    if ($rc)
      {
      print $trans_err_msg;
      return $rc;
      }
    } # END unless ($dryRun)

  # cntl-c during the scp password prompt will return 0
  print "\n\nThe file was successfully sent to Adaptive Computing support.\n"
    unless $dryRun;

  return;
  } # END sub interact_upload_file
#
#------------------------------------------------------------------------------
#
sub upload_file
  {
  my $expect_timeout_msg =
    "\n\nSorry, could not $trans_meth the file \"$tarBall\" to Adaptive Computing support.\nPlease send this file to your Adaptive Computing support contact.\n\n";

  # users should not send if they want xml but need to check
  print "Starting the file transfer to Adaptive Computing support ...\n\n"
    unless $useXML;

  # need the -v to get the pty to output the Exit status
  # so we know when scp has completed

  unless ($dryRun)
    {

    # stop the verbose scp msgs
    $Expect::Log_Stdout = 0;
    $exp = Expect->spawn ("scp -v $tarBall " . AC_SUPPORT);

    unless ($exp)
      {
      print $expect_timeout_msg;
      exit 1;
      }

    #$exp->debug(1);

    $exp->expect (10, 'password:');

    if ($exp->exp_error ())
      {
      print $expect_timeout_msg;
      exit 1;
      }

    $exp->send ("hello\n");
    $exp->expect (20, 'Exit status 0');

    if ($exp->exp_error ())
      {
      print $expect_timeout_msg;
      exit 1;
      }

    # perl verbose only once warning
    $Expect::Log_Stdout = 1;

    } # END unless ($dryRun)

  debugPrint (
    "\nThe file \"$tarBall\" has been sent to your Adaptive Computing support contact.\n\n"
  );

  print "... Completed the file transfer to Adaptive Computing support.\n"
    unless $useXML;

  } # END sub upload_file
#
#------------------------------------------------------------------------------
#
sub inp_file_trans
  {
  my $file_trans_meth;
  my $y_or_n;
  my $ans;

#   print
#     "If you have the Expect perl module installed I can upload the diagnostics tar file to Adaptive Computing.\n";
  print "Would you like to upload the tar file to Adaptive Computing support?\n";
  print "(Y/n)\n";
  chomp($y_or_n = <STDIN>);
  return 'no' if $y_or_n =~ /\s*[nN]/;

  if ($no_expect)
    {
    print
      "You answered yes to upload the file but you do not seem to have the Expect perl module installed.  If you install the Expect perl module then this script can automatically transfer the file.  For now, the script will try to transfer the file but you must enter the password when prompted.\n\n";
    print "Hit the Enter key to continue.";

    chomp($ans = <STDIN>);

    return '';
    } # END if ($no_expect)

  return 'scp';
  } # END sub inp_file_trans
#
#------------------------------------------------------------------------------
#
sub inp_ticket_num
  {
  my $tick_num;
  print "Please enter your Adaptive Computing Support ticket number (Optional).\n";
  chomp($tick_num = <STDIN>);

  return $tick_num;
  } # END sub inp_ticket_num
#
#------------------------------------------------------------------------------
#
sub gatherMoabDiag
{
    # Gather Moab run-time information
    runAndSaveDiag (
	            'showq --timeout=' . MOAB_CMDS_TIMEOUT . '  --about',
	            'Client Build Info',
	            'Moab:Workload'
	           );
    runAndSaveDiag (
	            'moab --timeout=' . MOAB_CMDS_TIMEOUT . '  --about',                    
	            'Moab Build Info and Version for moab binary',
	            'Moab:Workload'
	           );
    runAndSaveDiag (
	            'msub --timeout=' . MOAB_CMDS_TIMEOUT . ' --about',
	            'Version for qsub binary',
	            'Moab:Workload'
	           );
    runAndSaveDiag (
	            'checkjob --timeout=' . MOAB_CMDS_TIMEOUT . ' --about',
	            'Version for checkjob binary',
	            'Moab:Workload'
	           );
    runAndSaveDiag (
	            'changeparam --timeout=' . MOAB_CMDS_TIMEOUT . ' --about',
	            'Version for changeparam binary',
	            'Moab:Workload'
	           );
    runAndSaveDiag (
	            'mjobctl --timeout=' . MOAB_CMDS_TIMEOUT . ' --about',
	            'Version for mjobctl binary',
	            'Moab:Workload'
	           );
    runAndSaveDiag (
	            'mdiag --timeout=' . MOAB_CMDS_TIMEOUT . ' --about',
	            'Version for mdiag binary',
	            'Moab:Workload'
	           );
    runAndSaveDiag (
	            'mrsvctl --timeout=' . MOAB_CMDS_TIMEOUT . ' --about',
	            'Version for mrsvctl binary',
	            'Moab:Workload'
	           );
    runAndSaveDiag (
	            'mshow --timeout=' . MOAB_CMDS_TIMEOUT . ' --about',
	            'Version for mshow binary',
	            'Moab:Workload'
	           );
    runAndSaveDiag (
	            'mvmctl --timeout=' . MOAB_CMDS_TIMEOUT . ' --about',
	            'Version for mvmctl binary',
	            'Moab:Workload'
	           );
    runAndSaveDiag (
	            'runjob --timeout=' . MOAB_CMDS_TIMEOUT . ' --about',
	            'Version for runjob binary',
	            'Moab:Workload'
	           );
    runAndSaveDiag (
	            'showbf --timeout=' . MOAB_CMDS_TIMEOUT . ' --about',
	            'Version for showbf binary',
	            'Moab:Workload'
	           );
    runAndSaveDiag (
	            'showstart --timeout=' . MOAB_CMDS_TIMEOUT . ' --about',
	            'Version for showstart binary',
	            'Moab:Workload'
	           );
    #-------------------------------Running commands
    if ($moabIsRunning eq 0)
    {
	    runAndSaveDiag (
		            'mdiag --timeout=' . MOAB_CMDS_TIMEOUT . '  -S -v -v',
		            'Scheduler Diagnostics',
		            'Moab:General'
		           );
	    runAndSaveDiag (
		            'showstats --timeout=' . MOAB_CMDS_TIMEOUT . '  -v',
		            'Scheduler Stats',
		            'Moab:General'
		           );
	    runAndSaveDiag (
		            'mdiag --timeout=' . MOAB_CMDS_TIMEOUT . '  -R -v -v',
		            'Resource Manager Diagnostics',
		            'Moab:General'
		           );
	    runAndSaveDiag (
		            'showq --timeout=' . MOAB_CMDS_TIMEOUT . '  -v',
		            'Detailed Job Info',
		            'Moab:Workload'
		           );
	    runAndSaveDiag (
		            'showq --timeout=' . MOAB_CMDS_TIMEOUT . '  --blocking',
		            'Detailed Job Info',
		            'Moab:Workload'
		           );
	    runAndSaveDiag (
		            'mdiag --timeout=' . MOAB_CMDS_TIMEOUT . '  -p -v',
		            'Job Priority', 'Moab:Workload'
		           );
	    runAndSaveDiag (
		            'checkjob --timeout=' . MOAB_CMDS_TIMEOUT . '  -v ALL',
		            'Job Diagnostics',
		            'Moab:Workload'
		           );
	    runAndSaveDiag (
		            'mdiag --timeout=' . MOAB_CMDS_TIMEOUT . '  -n -v',
		            'Detailed Node Info',
		            'Moab:Resources'
		           );
	    runAndSaveDiag (
		            'mdiag --timeout=' . MOAB_CMDS_TIMEOUT . '  -c -v',
		            'Class/Queue Diagnostics',
		            'Moab:Credentials'
		           );
	    runAndSaveDiag (
		            'mdiag --timeout=' . MOAB_CMDS_TIMEOUT . '  -C',
		            'Moab Config Diagnostics',
		            'Moab:Credentials'
		           );
	    runAndSaveDiag (
		            'mdiag --timeout=' . MOAB_CMDS_TIMEOUT . '  -a',
		            'Account Diagnostics',
		            'Moab:Credentials'
		           );
	    runAndSaveDiag ('mdiag --timeout=' . MOAB_CMDS_TIMEOUT . '  -j -v',
		            'Job Diagnostics',
		            'Moab:Workload');
	    runAndSaveDiag (
		            'mdiag --timeout=' . MOAB_CMDS_TIMEOUT . '  -g',
		            'Group Diagnostics',
		            'Moab:Credentials'
		           );
	    runAndSaveDiag (
		            'mdiag --timeout=' . MOAB_CMDS_TIMEOUT . '  -G',
		            'Power Management',
		            'Moab:General'
		           );
	    runAndSaveDiag (
		            'mdiag --timeout=' . MOAB_CMDS_TIMEOUT . '  -q',
		            'QoS Diagnostics',
		            'Moab:Credentials'
		           );
	    runAndSaveDiag (
		            'mdiag --timeout=' . MOAB_CMDS_TIMEOUT . '  -u',
		            'Moab:Credentials'
		           );
	    runAndSaveDiag (
		            'mdiag --timeout=' . MOAB_CMDS_TIMEOUT . '  -t -v',
		            'Trigger Diagnostics',
		            'Moab:General'
		           );
	    runAndSaveDiag (
		            'mdiag --timeout=' . MOAB_CMDS_TIMEOUT . '  -T -v',
		            'Partition Diagnostics',
		            'Moab:General'
		           );
	    runAndSaveDiag (
		            'mdiag --timeout=' . MOAB_CMDS_TIMEOUT . '  -r -v',
		            'Reservation Diagnostics',
		            'Moab:Workload'
		           );
	    runAndSaveDiag (
		            'mdiag --timeout=' . MOAB_CMDS_TIMEOUT . '  -s -v',
		            'Standing Reservation Diagnostics',
		            'Moab:Workload'
		           );
	    runAndSaveDiag (
		            'mcredctl --timeout=' . MOAB_CMDS_TIMEOUT . '  -q policies user:ALL',
		            'Credential Policy',
		            'Moab:Workload'
		           );
	    runAndSaveDiag (
		            'showres --timeout=' . MOAB_CMDS_TIMEOUT . '  -v',
		            'Reservation Info',
		            'Moab:Workload'
		           );
	    runAndSaveDiag (
		            'showres --timeout=' . MOAB_CMDS_TIMEOUT . '  -n',
		            'Reservation-Nodes Info',
		            'Moab:Workload'
		           );
	    runAndSaveDiag (
		            'showconfig --timeout=' . MOAB_CMDS_TIMEOUT . '  -v',
		            'System Configuration',
		            'Moab:General'
		            );
	    runAndSaveDiag (
		            'showq --timeout=' . MOAB_CMDS_TIMEOUT . '  --loglevel=9',              
		            'Detailed Client Info',                                                 
		            'Moab:Workload'
		           );  
	    runAndSaveDiag (
		            'mdiag --timeout=' . MOAB_CMDS_TIMEOUT . '  -f -v',                     
		            'Fairshare Tree Diagnostics',                                           
		            'Moab:Workload'
		           );
	    runAndSaveDiag (
		            'mschedctl --timeout=' . MOAB_CMDS_TIMEOUT . '  -l vpc',                
		            'VPC Diagnostics',                                     
		            'Moab:Workload'
		           );
	    runAndSaveDiag (
		            'mvmctl --timeout=' . MOAB_CMDS_TIMEOUT . '  -q ALL',                   
		            'Standing Reservation Diagnostics',                                     
		            'Moab:Workload'
		           );
	    runAndSaveDiag (
		            'checknode --timeout=' . MOAB_CMDS_TIMEOUT . ' ALL',
		            'Check all nodes',
		            'Moab:Workload'
		           );
	    runAndSaveDiag (
		            'showq --timeout=' . MOAB_CMDS_TIMEOUT . ' -c',
		            'Show completed jobs',
		            'Moab:Workload'
		           );
	    runAndSaveDiag (
		            'mnodectl --timeout=' . MOAB_CMDS_TIMEOUT . ' -q wiki ALL',
		            'Wiki node information',
		            'Moab:Workload'
		           );
	    runAndSaveDiag (
		            'mdiag --timeout=' . MOAB_CMDS_TIMEOUT . ' -p -t',
		            'Partition priority output',
		            'Moab:Workload'
		           );
    }
}
#
#------------------------------------------------------------------------------
#
sub gatherMoabcfgAndLogs
{
    my $tmpLogDir;
    my $moabConfigOutput = `mdiag -C`;


    execCmd("mkdir $outDir/etc");
    execCmd("mkdir $outDir/msm");
    execCmd("mkdir $outDir/opt"); 

    # Copy configuration 
    if (-e "moab.dat") { execCmd("cp moab*.dat $outDir/"); }
    if (-e "spool/moab.cfg") { execCmd("cp spool/moab.cfg $outDir/moab_spool.cfg"); }
    execCmd("cp .moab.* $outDir/");
    execCmd("cp *.cfg $outDir/");

    if (-e "etc/") { execCmd("cp etc/ -r $outDir/"); }  #Copy etc/ dir
    if (-e "msm/") { execCmd("cp -r msm/  $outDir/"); }  #Copy legacy msm setup
    if (-e "/opt/mwm/") { execCmd("cp -r /opt/msm/ $outDir/opt/"); } #Copy msm if found under opt
    if (-e "tools/") {execCmd("cp -r tools/ $outDir/"); }  #Copy tools dir
    if (-e "$moabBinary") { execCmd("cp $moabBinary $outDir/"); } 



    # Copy optional log file
    if (! defined($includeLogLines) || ($includeLogLines < 0))
    {
        #Add new code to gather logging from the configured LOGDIR
	if ($moabConfigOutput =~ m/LOGDIR/i)
	{
            $moabConfigOutput =~ m/(LOGDIR\s*\/.*')/;
            $tmpLogDir = $1;
            $tmpLogDir =~ s/LOGDIR//;
            $tmpLogDir =~ s/\s*//;
            $tmpLogDir =~ s/'//;
            chomp($tmpLogDir);
 
            if (-e $tmpLogDir) { execCmd("cp $tmpLogDir -r $outDir/"); }
	}
        else
        {   
            #Gather logging from the default location if LOGDIR is not set. 
            execCmd("cp $logDir -r $outDir/");
        }
        
    }
    elsif (defined($includeLogLines) && ($includeLogLines > 0))
    {

        #Add new code to gather logging from the configured LOGDIR
        if ($moabConfigOutput =~ m/LOGDIR/i)
        {
            $moabConfigOutput =~ m/(LOGDIR\s*\/.*')/;
            $tmpLogDir = $1;
            $tmpLogDir =~ s/LOGDIR//;
            $tmpLogDir =~ s/\s*//;
            $tmpLogDir =~ s/'//;
            chomp($tmpLogDir);

            if (-e "$includeLogLines $tmpLogDir/moab.log") 
            {
                execCmd("tail -n $includeLogLines $tmpLogDir/moab.log > $outDir/moab.log.part");
            }
        }
        else
        {
            if (-e "$includeLogLines $logDir/moab.log") 
            {
                execCmd("tail -n $includeLogLines $logDir/moab.log > $outDir/moab.log.part");
            }
        }
    }
    # Copy other logfiles
    execCmd("cp $logDir/*query* $outDir/");
    execCmd("cp $logDir/partition* $outDir/");

}
#
#------------------------------------------------------------------------------
#
sub gatherViewpointDiag                                                                     
{                                                                                           
                                                                                               
   #Checks for Viewpoint env variable if any.                                                  
    my $viewpointHome =  `env | grep -i VIEWPOINT_HOME`;                                        
    my $viewpointPath = "/opt/viewpoint";                                                       
                                                                                               
        if ($viewpointHome){                                                                
            my @viewpointEnv = split(/=/, $viewpointHome);                                      
            $viewpointPath = $viewpointEnv[1];                                                  
            #Collect the homedir                                                                
            if (-e "$viewpointPath/") { execCmd("cp -r $viewpointPath/ $outDir/"); }
        }                                                                                   
        else{                                                                               
            if (-e "$viewpointPath/") { execCmd("cp -r $viewpointPath/ $outDir/"); }
        }                                                                                   
                                                                                               
   #Check for the viewpoint environment variable VIEWPOINT_HOME.                                             
   #If VIEWPOINT_HOME is there use it for the directory location. 
   #If VIEWPOINT_HOME is not in the path use /opt/viewpoint to
   #copy the contents of viewpoint home to our $outDir/                                       
   
   #Gather tomcat logs for version 5 and 6
   if(-e  "/var/log/tomcat6/catalina.out")
   {
       `cp /var/log/tomcat6/catalina.out $outDir/catalina.out`;
   }
   if(-e  "/var/log/tomcat5/catalina.out")
   {
       `cp /var/log/tomcat5/catalina.out $outDir/catalina.out`;
   }
 
}
#
#------------------------------------------------------------------------------
#
sub gatherSystemDiag
{
    # Gather operating system information
    runAndSaveDiag('uname -a',                     'OS Info',                'OS');         
    runAndSaveDiag('df',                           'Filesystem Info',        'Filesystem'); 
    runAndSaveDiag('ps -aux',                      'Running Processes',      'OS');         
    #Added by support
    runAndSaveDiag('vmstat 1 4',                   'System Load',            'OS');         
    runAndSaveDiag('date',                         'System Date',            'OS');         
    runAndSaveDiag('free -k',                      'System Memory',          'OS');         
    runAndSaveDiag('mount',                        'Mount Information',      'OS');         
    if (-e "/etc/mtab") 
    { 
      execCmd("cp /etc/mtab $outDir/"); 
    }
    if (-e "/etc/group") 
    {
      execCmd("cp /etc/group $outDir/"); 
    }
    if (-e "/etc/hosts") 
    {
      execCmd("cp /etc/hosts $outDir/");
    }       
    if (-e "/etc/sysconfig/network") 
    {
      execCmd("cp /etc/sysconfig/network $outDir/");    
    }
    if (-e "/etc/hosts.equiv") 
    {
      execCmd("cp /etc/hosts.equiv $outDir/");    
    } 
    if (-e "/etc/odbc.ini") 
    { 
      execCmd("cp /etc/odbc.in $outDir/");   
    }    
    if (-e "/etc/odbcinst.ini") 
    { 
      execCmd("cp /etc/odbcinst.ini $outDir/");   
    }         
    if (-e "/etc/sysconfig/selinux") 
    {
      execCmd("cp /etc/sysconfig/selinux $outDir/");     
    }         
    runAndSaveDiag('chkconfig --list',             'Chkconfig services',     'OS');
    runAndSaveDiag('uptime',                       'Server uptime',          'OS');
    runAndSaveDiag('route',                        'Network Route',          'OS');
    runAndSaveDiag('iptables-save',                'Iptables export',        'OS');
    if (-e "/etc/network/interfaces")
    {
       execCmd("cp /etc/network/interfaces $outDir/");
    }
    if (-e "/etc/sysconfig/network-scripts/")
    {
       execCmd("cp /etc/sysconfig/network-scripts/ -r $outDir/");
    }
    runAndSaveDiag('ip addr show',                 'IP address information', 'OS');
    execCmd("cp /etc/*release* $outDir/");
    runAndSaveDiag('ifconfig',                     'OS Ifconfig',            'OS');
    runAndSaveDiag('netstat',                      'Netstat output',         'OS');
    runAndSaveDiag('service --status-all',         'Service status',         'OS');
    runAndSaveDiag('rpm -qa',                      'For rpm based systems',  'OS'); 
}
#
#------------------------------------------------------------------------------
#
sub gatherSlurmDiag
{
    if (-e "/usr/local/etc/slurm.conf")
    {
      execCmd("cp /usr/local/etc/slurm.conf $outDir/");
    }
    if (-e "/usr/local/etc/wiki.conf")
    {
      execCmd("cp /usr/local/etc/wiki.conf $outDir/");
    }
    runAndSaveDiag('scontrol show jobs',               'SLURM show jobs',         'SLURM');
    runAndSaveDiag('sinfo -v',                         'SLURM Node info',         'SLURM');
    runAndSaveDiag('squeue -l -v',                     'SLURM detailed job info', 'SLURM');
    runAndSaveDiag('scontrol show partition',          'SLURM partition info',    'SLURM');
    runAndSaveDiag('scontrol show config',             'SLURM configuration',     'SLURM');
}
#
#------------------------------------------------------------------------------
#
sub gatherMamDiag
{
    if (-e "/opt/mam")
    {
       execCmd("cp /opt/mam/ -r $outDir/");
    }
    if (-e "/opt/mam")
    {
       execCmd("cp /opt/mam/ -r $outDir/");
    }
}
#
#------------------------------------------------------------------------------
#
sub gatherMwsDiag
{
   if (-e "/opt/mws")
   {
      execCmd("cp /opt/mws/ -r $outDir/");
   }
}
#
#------------------------------------------------------------------------------
#
sub gatherTorqueDiag
{
    runAndSaveDiag('pbsnodes -a',        'TORQUE Node Info',   'TORQUE');
    runAndSaveDiag('qstat',              'TORQUE Queue Info',  'TORQUE');
    runAndSaveDiag('qstat -f',           'TORQUE Job Info',    'TORQUE');
    runAndSaveDiag('qstat -q',           'TORQUE Queue Info',  'TORQUE');
    runAndSaveDiag('qmgr -c "p s"',      'TORQUE Server Info', 'TORQUE');
    runAndSaveDiag('qstat --about',      'TORQUE client Info', 'TORQUE');
    runAndSaveDiag('pbs_server --about', 'TORQUE Server Info', 'TORQUE');
}
#
#------------------------------------------------------------------------------
#
sub gatherTorqueLogs
{

    #Path to proc and combine with the pid of pbs_server. Next copy the logs from that pid.
    my $torque_log_dir ="/proc/";
    chomp ($torque_log_dir .= `pgrep -f pbs_server`);
    $torque_log_dir .= "/fd/4";
    
    #Make sure the file is there
    if (-e $torque_log_dir)
    {
        `cp $torque_log_dir  $outDir/torque_logs `;
    }
    
    #Now gather trqauthd logs
    $torque_log_dir ="/proc/";
    chomp ($torque_log_dir .= `pgrep -f trqauthd`);
    $torque_log_dir .= "/fd/3";

    #Make sure the file is there
    if (-e $torque_log_dir)
    {
        `cp $torque_log_dir  $outDir/trqauthd_logs `;
    }
    

}
#
#------------------------------------------------------------------------------
#
sub gatherPerlDiag
{
# Verify that Perl scripts are in good standing (this helps for diagnosing data-staging problems, etc.)
    runAndSaveDiag('perl -v', 'Perl Version', 'Moab:Tools');

    my @perlFiles = glob "$FindBin::Bin/*.pl";
    foreach my $file (sort @perlFiles)
    {
        runAndSaveDiag(
            "perl -wc $file",  "Check ${file}'s Dependencies",
            'Moab:Tools:Perl', 'perlTools.txt'
        );
    }
}
#
#------------------------------------------------------------------------------
#
sub bundlePackage
{
    # Bundle files together
    if (execCmd("tar -zcf $tarBall $outDir") != 0)
    {
        die("ERROR: could not create $tarBall bundle!\n");
    }
    execCmd("rm -rf $outDir");
}
#
#------------------------------------------------------------------------------
#
sub checkForBacktraces
{
    # Check log files for backtrace information.
    my $command = "grep -h  -A 100 -e '----Begin Moab Backtrace---' $logDir/moab.log*";
    my $printing = 0;

    open (INDATA, "-|", $command)  or die "Couldn't execute program: $!";
    open (OUTDATA, "> $outDir/backtraces")  or die "Can't open file: $!";

    while ( defined( my $line = <INDATA> )  ) {
      #chomp($line);
      if ($line =~ m/---Begin Moab Backtrace--/) {$printing = 1;}
      # skip interspersed log lines.
      next if $line =~ m#^\d\d/\d\d \d\d:\d\d:\d\d#;
      if ($printing) 
      {
        # line could look like:
        # 
        #/opt/moab/bin/moab(main+0x673)[0x43e3e7]
        #/opt/moab/bin/moab[0x406277]
        # 
        # need to separate the filename and the address (parenthesized stuff is optional)
        if ($line =~ m#^(/.*?)(\(.+\))*\[(.*)\]#)
        {
          my $prettyline = `addr2line -f --demangle -e $1 $3`;
          # addr2line may not be present on the system 
          # in that case use the default (no line number) trace.
          if (not defined $prettyline) {$prettyline = $line;}
          # join the 2 line output to one.
          if ( $prettyline =~ m/^(.+?)\n(.*)$/ )
          {
            $prettyline = sprintf("%32s | %s\n", $1, $2);
          }
          print OUTDATA $prettyline;
        }
        else
        {
          print OUTDATA $line;
        }
      }
      if ($line =~ m/---End Moab Backtrace--/) {$printing = 0;}
   # print "$line\n";
    }

    close OUTDATA;
    close INDATA;
}
#
#==============================================================================
#
sub runAndSaveDiag
{
    my ($cmd, $desc, $category, $filename) = @_;
    $filename = $filename || $cmd;
    $filename =~ s/--timeout=\d+\s*//;
    $filename =~ tr{ /}{_-};
    print "FILENAME: $filename\n";

    # Check command list & parameters
    $cmdCount++;
    if (scalar(@commandList) > 0)
    {
        # skip if command not present in allowed list
        return unless grep {/^$cmdCount$/} @commandList;
    }
    if ($useXML && $dryRun)
    {
        printCommandXML(@_);
        return;
    }

    # Run command and save as file in $outDir
    #print "Executing: '$cmd' and saving in '$outDir'\n" if $logLevel >= 1;

    my $output = '-' x 27 . "\nResults for command: $cmd\n\n";
    unless ($dryRun)
    {
        $output .= `$cmd 2>&1`;
        if ($?)
        {
            print "Info: command '$cmd' did not successfully complete!\n"
              if $logLevel >= 1;
            print
              qq|<status Id="$cmdCount"><error Message="error occurred"></error></status>|
              if $useXML;
            $numErrors++;
        }
    }

    my $outputFilename = "$outDir/$filename.txt";
    print "Saving to file: >>$outputFilename\n" if $logLevel >= 2;
    unless ($dryRun)
    {
        open my $fhandle, ">>$outputFilename"
          or die "Cannot open Output File ($outputFilename): $!";
        print $fhandle $output;
        close $fhandle;
    }

    print qq|<status Id="$cmdCount"><success></success></status>| if $useXML;
}
#
#------------------------------------------------------------------------------
#
sub checkForPoundInclude
{
    #Added to capture #include files from the moab.cfg file
    if (-e "moab.cfg") { $moab_pound = "moab.cfg"; capturePoundInclude() }
    if (-e "etc/moab.cfg") { $moab_pound = "etc/moab.cfg"; capturePoundInclude()  }
    if (-e "spool/moab.cfg") { $moab_pound = "spool/moab.cfg"; capturePoundInclude() }

}
#
#------------------------------------------------------------------------------
#
sub capturePoundInclude
{
    if ($moab_pound ne "0") {
        open FILE, "$moab_pound" or die $!;
        my $fileOutput;
        my $moab_line;
        my @lines;
        @lines = <FILE>;
        close (FILE);
        foreach $moab_line (@lines)
        {
            if ($moab_line =~ m/^#INCLUDE/i){
                $moab_line =~ s/#INCLUDE //i;
				chomp($moab_line);
                if (-e "$moab_line") { execCmd("cp $moab_line $outDir/"); }
            }
        }
    }
}
#
#------------------------------------------------------------------------------
#
sub execCmd
{
    my $cmd = $_[0];
    print "Executing: $cmd\n" if $logLevel >= 1;
    return 0                  if $dryRun;

    `$cmd 2>&1`;
    if ($? != 0)
    {
          #if $logLevel >= 1;
        $numErrors++;
        return -1;
    }

    return 0;
}
#
#------------------------------------------------------------------------------
#
sub debugPrint
{
    print $_[0] if (! $useXML && ($logLevel >= 1));
}
#
#------------------------------------------------------------------------------
#
sub printCommandXML
{
    my ($cmd, $desc, $category) = @_;
    $desc     = '' unless $desc;
    $category = '' unless $category;

    print '<commands>' if ($cmdCount == 0);
    print
      qq|<command Id="$cmdCount" Name="$cmd" Desc="$desc" Category="$category"></command>|;
}
#
###############################################################################

1;

__END__

=head1 NAME

=head1 SYNOPSIS

=head1 DESCRIPTION

=head2 EXPORT

=head2 EXPORT_OK

=head1 SEE ALSO

Additional information can be found at http://www.adaptivecomputing.com/

=head1 AUTHOR

Adaptive Computing, Inc., E<lt>http://www.adaptivecomputing.com/E<gt>

=head1 COPYRIGHT AND LICENSE

Copyright (C) 2005-2016 by Adaptive Computing Enterprises, Inc. All Rights Reserved.

=cut
