#!/usr/bin/python
  
import sys, termios, atexit
import zmq
import os
import time
import argparse
import base64
import hashlib
from Crypto.Cipher import AES
from time import sleep
from select import select

# list element structure:
# [ FunctionName, Min, Max, Avg, Total, Count, Last ]

FunctionList = [];
PrintDirtyFlag = False;
LastPrintTime = time.time();

#-----------------------------------------------
# Keyboard functions
#-----------------------------------------------

# save the terminal settings
fd = sys.stdin.fileno()
new_term = termios.tcgetattr(fd)
old_term = termios.tcgetattr(fd)

# new terminal setting unbuffered
new_term[3] = (new_term[3] & ~termios.ICANON & ~termios.ECHO)

# switch to normal terminal
def set_normal_term():
    termios.tcsetattr(fd, termios.TCSAFLUSH, old_term)

# switch to unbuffered terminal
def set_curses_term():
    termios.tcsetattr(fd, termios.TCSAFLUSH, new_term)

def getch():
    return sys.stdin.read(1)

def kbhit():
    dr,dw,de = select([sys.stdin], [], [], 0)
    return dr <> []

#-----------------------------------------------
# UpdateTimes
#-----------------------------------------------

def UpdateTimes(node, time):
    Total = node[4] + time
    Count = node[5] + 1

    # calculate average
    node[3] = Total / Count

    # Check min/max
    if time < node[1] or node[1] == 0:
        node[1] = time
    if time > node[2]:
        node[2] = time

    # Update total, count, and last
    node[4] = Total
    node[5] = Count
    node[6] = time


#-----------------------------------------------
# AddData
#-----------------------------------------------

def AddData(function, time):
    global PrintDirtyFlag

    foundFunction = False
    # check and see if the function is already in the function list
    nodeIndex = 0
    for node in FunctionList:
        if node[0] == function:
            # update the time values
            foundFunction = True
            UpdateTimes(node, time)
            break
        elif function < node[0]:
            # insert the function in sorted order before the existing node
            FunctionList.insert(nodeIndex, [function, time, time, time, time, 1, time])
            foundFunction = True
            break

        nodeIndex = nodeIndex + 1

    if foundFunction == False:
        # create a new node
        FunctionList.append([function, time, time, time, time, 1, time])

    PrintDirtyFlag = True

#-----------------------------------------------
# GetFunctionName
#-----------------------------------------------

def GetFunctionName(line):
    sections = line.split(",")
    for section in sections:
        if "Function" in section:
            return section.split(":")[1]

#-----------------------------------------------
# GetDuration
#-----------------------------------------------

def GetDurationText(line):
    sections = line.split(",")
    for section in sections:
        if "Duration" in section:
            tuples = section.split(":")
            return tuples[1].split("\"")[0]

#-----------------------------------------------
# ResetFunctionList
#-----------------------------------------------

def ResetFunctionList():
    for node in FunctionList:
        node[1] = 0;
        node[2] = 0;
        node[3] = 0;
        node[4] = 0;
        node[5] = 0;
        node[6] = 0;

    PrintList(True)

#-----------------------------------------------
# PrintList
#-----------------------------------------------

def PrintList(override = False):
    global PrintDirtyFlag
    global LastPrintTime

    timeNow = time.time()
    if (override == True or (PrintDirtyFlag == True and ((timeNow - LastPrintTime) >= 0.5))):
        nodeCount = 0
        for node in FunctionList:
            print "%-25s %11s %11s %11s %11s %11s %13s" % (node[0], int(node[1]), int(node[2]), int(node[3]), int(node[6]), int(node[5]), int(node[4]))
            nodeCount = nodeCount + 1

        for n in range(0, nodeCount):
          sys.stdout.write("\x1b[A") # Move the cursor up one

        PrintDirtyFlag = False
        LastPrintTime = timeNow

#-----------------------------------------------
# Main
#-----------------------------------------------

# get command line arguments
parser = argparse.ArgumentParser(description="Print Moab function timing")
parser.add_argument("host", help="Hostname of Moab server to subscribe to")
parser.add_argument("-k", "--key",
    help="Decrypt using the MESSAGEQUEUESECRETKEY in the moab-private.cfg file. Example: cWJk+vQckTAL26jX64Qq3Q==")
args = parser.parse_args()

key = None
initialization_vector = None
unpad = None

if args.host is None:
    print "Error: No host name specified on command line."
    quit()

if args.key is not None:
    key = base64.b64decode(args.key)
    initializationVector = hashlib.md5(key).digest()
    unpad = lambda s: s[0:-ord(s[-1])]

# clear and start printing
os.system('clear');

print("Function                          Min         Max     Average        Last       Count    Total time")
print("------------------------- ----------- ----------- ----------- ----------- ----------- -------------")

atexit.register(set_normal_term)
set_curses_term()

context = zmq.Context()
sock = context.socket(zmq.SUB)
sock.setsockopt(zmq.SUBSCRIBE, 'telemetry.timing')
sock.connect('tcp://' + args.host + ':5566')

while True:
    try:
        if kbhit():
            ch = str(getch())
            if (ch == "c" or ch == "C" or ch == 'z' or ch == 'Z'):
                # clear the array
                ResetFunctionList()
            elif (ch == "q" or ch == "Q" or ch == 'x' or ch == 'X'):
                os.system('clear');
                break

        message = sock.recv(zmq.NOBLOCK)

        if 'telemetry.timing' in message:
            continue

        # check to see if the message needs to be decrypted
        if key is not None:
            decrypter = AES.new(key, AES.MODE_CBC, initializationVector)
            message = base64.b64decode(message)
            message = unpad(decrypter.decrypt(message))
            #print message

            # check message for content
            if "messageType" not in message:
                print "Error: key is incorrect."
                quit()

        # if messages are coming fast, then only print the list once every second or two
        # get the current time, and if

        if "MTimer.c" in message:
            lines = message.splitlines()
            for line in lines:
                if "Duration" in line:
                    AddData(GetFunctionName(line), int(GetDurationText(line)))
                    PrintList()

    except (zmq.ZMQError):
        # wait for 200ms before trying again
        PrintList()
        sleep(0.2)
        pass
