#! /usr/bin/perl -w
################################################################################
#
# Usage: event.create.sql.pl
#
# This script is used to log event records to an sql database.
#
# It accepts one or more event records on STDIN of the format:
# <human readable time> <event epoch time> <object type> <object id> <object event type> <node count> <task count> <user name> <group name> <wallclock limit> <job completion state> <class name> <submit time> <dispatch time> <ecompletion time> <network adapter> <architecture>
#
# It writes these fields to an sql database table called event_log with fields:
# m_timestamp, m_object_type, m_object_id, m_object_event, m_node_count, m_task_count, m_user_name, m_group_name, m_wallclock_limit, m_end_state, m_class_name, m_submit_time, m_start_time, m_end_time, m_memory, m_disk, m_qos, m_account
#
#   Copyright (C) 2006-2016 by Adaptive Computing Enterprises, Inc. All Rights Reserved.
#
################################################################################
#use strict;
use FindBin;
use lib "/opt/moab/lib/perl5", "/opt/moab/etc";
use lib "${FindBin::Bin}/../../lib/perl5", "${FindBin::Bin}/../../etc";
use Getopt::Long;
use DBI;
use Moab::Tools;
BEGIN { require "config.sql.pl"; }
our ($logLevel, $datasource, $db_table, $db_user, $db_password);

logPrint("$0 ", join(' ', @ARGV), "\n") if $logLevel;

# Parse Command Line Arguments
my ($help);
my $usage = "Usage: $0\n";
GetOptions('help|?' => \$help) or logDie($usage);
die $usage if $help;

# Connect with the database
my $dbuser   = $db_user || (getpwuid($<))[0];
my $dbpasswd = $db_password;
my $dbhandle =
  DBI->connect($datasource, $dbuser, $dbpasswd,
    {AutoCommit => 1, RaiseError => 1});

# Read in the event records from standard in and write them to the database
my $count = 0;
foreach my $line (<STDIN>)
{
    logPrint($line) if $logLevel >= 2;
    chomp $line;
    my (
        $hRTime,         $timestamp,       $objectType,
        $objectId,       $objectEvent,     $nodeCount,
        $taskCount,      $userName,        $groupName,
        $wallclockLimit, $endState,        $className,
        $submitTime,     $dispatchTime,    $startTime,
        $endTime,        $network,         $architecture,
        $opsys,          $memcmp,          $memory,
        $diskcmp,        $disk,            $systemQueueTime,
        $allocatedTasks, $TPN,             $qosName,
        $flags,          $accountName,     @rest
      )
      = split /\s+/, $line;
    next unless defined $nodeCount;
    my $sql =
      "INSERT INTO $db_table (m_timestamp,m_object_type,m_object_id,m_object_event,m_node_count,m_task_count,m_user_name,m_group_name,m_wallclock_limit,m_end_state,m_class_name,m_submit_time,m_start_time,m_end_time,m_memory,m_disk,m_qos,m_account) VALUES ($timestamp,'$objectType','$objectId','$objectEvent','$nodeCount','$taskCount','$userName','$groupName','$wallclockLimit','$endState','$className','$submitTime','$startTime','$endTime','$memory','$disk','$qosName','$accountName')";
    logPrint($sql) if $logLevel >= 2;
    $count += $dbhandle->do($sql);
}

# Consider successful if inserted positive number of rows
if ($count)
{
    logWarn("Successfully inserted $count event log entries\n") if $logLevel;
}
else
{
    logDie("Failed to insert any log entries: $!\n");
}
exit 0;
1;

__END__

=head1 NAME

=head1 SYNOPSIS

=head1 DESCRIPTION

=head2 EXPORT

=head2 EXPORT_OK

=head1 SEE ALSO

Additional information can be found at http://www.adaptivecomputing.com/

=head1 AUTHOR

Adaptive Computing, Inc., E<lt>http://www.adaptivecomputing.com/E<gt>

=head1 COPYRIGHT AND LICENSE

Copyright (C) 2006-2016 by Adaptive Computing Enterprises, Inc. All Rights Reserved.

=cut
